/*
	File: ColorMungerUIWin.c

	Copyright 1996, Adobe Systems Incorporated.
	All rights reserved.

	C source file for Windows specific code for ColorMunger example.
*/

#include "ColorMunger.h"

/****************************************************************************/

short Validate (HWND dp, short *item, double *dx, Boolean useAlert);
void UpdateDialogInfo (GPtr globals, HWND dp);
BOOL WINAPI FilterProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam);
OSErr DoConversion (GPtr globals, HWND dp, ColorServicesInfo *csinfo);

/*****************************************************************************/

void DoAbout (GPtr globals)
{
	ShowAbout((AboutRecordPtr)gStuff, hDllInstance, AboutID);
}

/*****************************************************************************/
// Loops through values and automatically grabs and validates item.  Returns
// noErr if okay, otherwise focus has been forced back to invalid item.

short Validate(HWND dp, short *item, double *dx, Boolean useAlert)
{
	short 	numberErr = noErr;
	long 	x = 0; // returned long
	short 	loop = 1;
	
	const double propDoubles[] = {
		0, 0, 0
		/* iPropBigNudgeH, kNudgeMin, kNudgeMax,
		iPropBigNudgeV, kNudgeMin, kNudgeMax,
		iPropRulerOriginH, kOriginMin, kOriginMax,
		iPropRulerOriginV, kOriginMin, kOriginMax,
		iPropGridMajor, kGridMin, kGridMax */
		};

	const short	numPropDoubles = 
		(propDoubles[0] > 0) ? ((sizeof(propDoubles) / sizeof(double)) / 3) : 0;

	const long	propNumbers[] = {
		dESource1, kMin, kMax,
		dESource2, kMin, kMax,
		dESource3, kMin, kMax,
		dESource4, kMin, kMax,
		};

	const short	numPropNumbers = 
		(propNumbers[0] > 0) ? ((sizeof(propNumbers) / sizeof(long)) / 3) : 0;
	
	const short foundIt = -1; // flag that we found it

	
	// first loop through doubles, looking for match
	while (loop <= numPropDoubles && loop != foundIt)
	{
		if (propDoubles[loop*3-3] == *item)
		{
			numberErr = FetchDouble (dp,
									 *item,
									 propDoubles[loop*3-2], // max
									 propDoubles[loop*3-1], // min
									 dx); // returned double
			if (numberErr != noErr && useAlert)
			{ // out of bounds problem.  Report and change focus back.
				AlertDouble (dp,
							 *item,
							 propDoubles[loop*3-2], // max
							 propDoubles[loop*3-1], // min
							 dx, // value
							 hDllInstance, // global ref for win
							 AlertID, // global alert dialog
							 numberErr); // error we found
				*item = 0;
			}
			loop = foundIt; // found it!
		} else loop++; // next
	}

	if (loop != foundIt) loop = 1; // reset
	// now loop through longs, looking for match
	while (loop <= numPropNumbers && loop != foundIt)
	{
		if (propNumbers[loop*3-3] == *item)
		{
			numberErr = FetchNumber (dp,
									 *item,
									 propNumbers[loop*3-2], // max
									 propNumbers[loop*3-1], // min
									 &x); // returned long
			if (numberErr != noErr && useAlert)
			{ // out of bounds problem.  Report and change focus back.
				AlertNumber (dp,
							 *item,
							 propNumbers[loop*3-2], // max
							 propNumbers[loop*3-1], // min
							 &x, // value
							 hDllInstance, // global ref for win
							 AlertID, // global alert dialog
							 numberErr); // error we found
				*item = 0;
			}
			*dx = (double)x; // coerce long as returned double
			loop = foundIt; // found it!
		} else loop++; // next
	}
	return numberErr;
}

/*******************************************************************/
void UpdateDialogInfo (GPtr globals, HWND dp)
{
	short loop = 0;
	
	SetRadioGroupState (dp, dSource1, dSourceEnd, dSource1 + gColor);
	SetRadioGroupState (dp, dTarget1, dTargetEnd, dTarget1 + gColorize);

	for (loop = 0; loop < 4; loop++)
		StuffNumber(dp, dEditSource1 + loop, gSourceColor[loop]);
	for (loop = 0; loop < 4; loop++)
		StuffNumber(dp, dEditTarget1 + loop, gTargetColor[loop]);
}

/*******************************************************************/
OSErr DoConversion (GPtr globals, HWND dp, ColorServicesInfo *csinfo)
{
	OSErr	gotErr = noErr;
	short 	loop = 0;
	
	PopulateColorServicesInfo (globals, csinfo);
	gotErr = ColorServices (csinfo);
	gColorize = csinfo->resultSpace; // should be the same
	CopyColor (&csinfo->colorComponents, &gTargetColor);
	for (loop = 0; loop < 4; loop++)
		StuffNumber(dp, dEditTarget1 + loop, gTargetColor[loop]);

	return gotErr;
}
/*****************************************************************************/

BOOL WINAPI FilterProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)       // Win32 Change
{
	long				x = 1;
	double				dx = 0;
	short				numberErr = noErr;
	short				item;
	static short		lastItem = 0, lastTitem = 0;
	static short		cColor;
	static int16		cSourceColor[4];
	static short		cColorize;
	static int16		cTargetColor[4];
	static short		gotErr = 0;
	PIType				key = NULLID;
	static ColorServicesInfo	csinfo;
	int					cmd;
	static GPtr			globals = NULL;		  /* need to be static */
	
	switch  (wMsg)
	{

		case  WM_INITDIALOG:

			/* set up globals	*/

			globals = (GPtr) lParam;

			CenterDialog(hDlg);
			
			// save globals in locals for later if user cancels
			CopyColor(&gSourceColor, &cSourceColor);
			CopyColor(&gTargetColor, &cTargetColor);
			cColor = gColor;
			cColorize = gColorize;
					
			UpdateDialogInfo(globals, hDlg);

			SelectTextItem(hDlg, dEditSource1);

			/* drop into PAINT */
		case WM_PAINT:
			// paint here
			return FALSE;
			break;

		case  WM_COMMAND:
	  		item = COMMANDID (wParam);              // WIN32 Change
			cmd = HIWORD (wParam);

			switch  (item)
			{
				case dSourcePicker:
					if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
					{ // didn't have to change focus.  Continue.
						if (cmd == BN_CLICKED)
						{
							PopulateColorServicesInfo (globals, &csinfo);
							csinfo.selector = plugIncolorServicesChooseColor; // change
							csinfo.resultSpace = plugIncolorServicesChosenSpace; // will return user chosen
							
							if (gotErr = ColorServices (&csinfo) == noErr)
							{ // no problems.  Do this.
								gColor = csinfo.resultSpace;
								CopyColor (&csinfo.colorComponents, &gSourceColor);
								DoConversion(globals, hDlg, &csinfo);
								UpdateDialogInfo (globals, hDlg);
							}
							SelectTextItem(hDlg, lastTitem);
							return FALSE;
						}
					}
					break;
				case dTargetPicker:
					if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
					{ // didn't have to change focus.  Continue.
						if (cmd == BN_CLICKED)
						{
							PopulateColorServicesInfo (globals, &csinfo);
							csinfo.selector = plugIncolorServicesChooseColor; // picker
							if (gotErr = ColorServices (&csinfo) == noErr)
							{ // no problems.  Do this.
								// move old target to source space
								CopyColor (&gTargetColor, &gSourceColor);
								gColor = gColorize;
								// now pull results to target
								gColorize = csinfo.resultSpace; // should be the same
								CopyColor (&csinfo.colorComponents, &gTargetColor);
								UpdateDialogInfo (globals, hDlg);
							}
							SelectTextItem(hDlg, lastTitem);
							return FALSE;
						}
					}
					break;
				case ok:
					if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
					{ // validated fine
						if (cmd == BN_CLICKED)
						{ // wants to dismiss.  End dialog.
							EndDialog(hDlg, item); 
							return TRUE;
						}
					}
					break;
				case cancel:
					if (cmd == BN_CLICKED)
					{ // return all globals from saved locals
						CopyColor(&cSourceColor, &gSourceColor);
						CopyColor(&cTargetColor, &gTargetColor);
						gColor = cColor;
						gColorize = cColorize;

						gResult = userCanceledErr;
						EndDialog(hDlg, item);          // WIN32 change
						return TRUE;
					}
					// don't ever validate during cancel
					break;
				default:
					if (item >= dSource1 && item <= dSourceEnd)
					{
						if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
						{ // validated fine
							if (cmd == BN_CLICKED)
							{ // process button
								SetRadioGroupState (hDlg, dSource1, dSourceEnd, item);
								gColor = item - dSource1;
						
								DoConversion (globals, hDlg, &csinfo);
								SelectTextItem(hDlg, lastTitem);
								return FALSE;
							}
						}
					}
					else if (item >= dTarget1 && item <= dTargetEnd)
					{ // set group then do conversion
						if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
						{ // validated fine
							if (cmd == BN_CLICKED)
							{ // process							
								SetRadioGroupState (hDlg, dTarget1, dTargetEnd, item);
								gColorize = item - dTarget1;
								
								DoConversion (globals, hDlg, &csinfo);
								SelectTextItem(hDlg, lastTitem);
								return FALSE;
							}
						}
					}
					else if (item >= dEditSource1 && item <= dEditSourceEnd)
					{
						if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
						{
							if (cmd == EN_KILLFOCUS)
								lastItem = item; // validate will get this
							else if (cmd == EN_CHANGE)
							{ // do me
								numberErr = Validate(hDlg, &item, &dx, false);
								gSourceColor[item - dEditSource1] = (int16)dx;
								lastTitem = item;

								DoConversion (globals, hDlg, &csinfo);
								return FALSE;
							}
						}
					}
					break;
			} // switch (item)
			break; // case WM_COMMAND
		default:
			return FALSE;
			break;
	} // switch
	return  TRUE;
}

/*****************************************************************************/
short PromptUserForInput (GPtr globals)
{

	int    nResult;
	PlatformData *platform;

	platform = ((FilterRecordPtr) gStuff)->platformData;

	/* Query the user for percent dissolve. */

	nResult = DialogBoxParam(hDllInstance,
							 (LPSTR)"COLORMUNGERPARAM",
							 (HWND)platform->hwnd,
							 (FARPROC)FilterProc,
							 (LPARAM)globals);

	return (nResult == ok);

}	

/****************************************************************************/
/* Example for ShowAlert() function which takes a string ID as parameter    */
/* and displays a message box                                               */
/****************************************************************************/

short ShowAlert (short stringID)
{
	char szMessage[256];
	char szTitle[128];

	LoadString(hDllInstance, stringID, szMessage, sizeof szMessage);
	LoadString(hDllInstance, 2, szTitle, sizeof szTitle);
	return  ( MessageBox((HWND)NULL, (LPSTR)szMessage, (LPSTR)szTitle, MB_OK | MB_ICONHAND) );

}

/*****************************************************************************/

/* Initialization and termination code for window's dlls. */

/* ------------------------------------------------
 *   Code from Borland's window's dll example code.
 * ------------------------------------------------
 */
#if defined(__BORLANDC__)
// Turn off warning: Parameter '' is never used; effects next function only
#pragma argsused
#endif

#ifdef WIN32

// Every 32-Bit DLL has an entry point DLLInit

BOOL APIENTRY DLLInit(HANDLE hInstance, DWORD fdwReason, LPVOID lpReserved)
{

	if (fdwReason == DLL_PROCESS_ATTACH)
		hDllInstance = hInstance;

	return TRUE;   // Indicate that the DLL was initialized successfully.
}

#else
// Every 16-Bit DLL has an entry point LibMain and an exit point WEP.
int FAR PASCAL LibMain( HANDLE hInstance, WORD wDataSegment,
								   WORD wHeapSize, LPSTR lpszCmdLine )
{
	// Required when using Zortech; causes blink to include startup code
	extern __acrtused_dll;

	// The startup code for the DLL initializes the local heap (if there is one)
	// with a call to LocalInit which locks the data segment.
	if ( wHeapSize != 0 )
		UnlockData( 0 );

	hDllInstance = hInstance;
	return 1;   // Indicate that the DLL was initialized successfully.
}

int FAR PASCAL WEP(int nParam)
{
	switch  (nParam) {
	  case  WEP_SYSTEM_EXIT: // System shutdown in progress
	  case  WEP_FREE_DLL   : // DLL use count is 0
	  default :              // Undefined;  ignore
			return  1;
	}
}
#endif
/*****************************************************************************/


