/*
	File: ColorMunger.c

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	C source file for Propetizer example.
*/

#if __MWERKS__
#include <SetupA4.h> // A4-globals
#include <A4Stuff.h> // A4-globals
#endif

#if defined(THINK_C) || defined(__MWERKS__)
#define ENTRYPOINT main
#endif

#include "ColorMunger.h"

Handle hDllInstance = NULL;

/*****************************************************************************/

void InitGlobals (GPtr globals);
void DoParameters (GPtr globals);
void DoPrepare (GPtr globals);
void DoStart (GPtr globals);
void DoContinue (GPtr globals);
void DoFinish (GPtr globals);

/*****************************************************************************/

/* All calls to the plug-in module come through this routine. It must be
   placed first in the resource. To achieve this, most development systems
   require that this be the first routine in the source. */

#if MSWindows
	   void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result)				
#else
pascal void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result)				
#endif
	{
	
	Globals globalValues;
	GPtr globals = &globalValues;
	
	#if __MWERKS__
	EnterCodeResource(); // A4-globals
	#endif
	
	if (!*data)
		{
		
		InitGlobals (globals);

		*data = (long) NewHandle (sizeof (Globals));
		
		if (!*data)
			{
			*result = memFullErr;
			return;
			}
			
		** (GHdl) *data = globalValues;
		
		}
		
	globalValues = ** (GHdl) *data;
		
	gStuff = filterParamBlock;
	gResult = noErr;
		
	switch (selector)
		{
		
		case filterSelectorAbout:
			DoAbout (globals);
			break;
			
		case filterSelectorParameters:
			DoParameters (globals);
			break;
			
		case filterSelectorPrepare:
			DoPrepare (globals);
			break;
		
		case filterSelectorStart:
			DoStart (globals);
			break;
		
		case filterSelectorContinue:
			DoContinue (globals);
			break;
		
		case filterSelectorFinish:
			DoFinish (globals);
			break;
			
		default:
			gResult = filterBadParameters;
		
		}
		
	*result = gResult;
	** (GHdl) *data = globalValues;

	#if __MWERKS__
	ExitCodeResource(); // A4-globals
	#endif
}

/*****************************************************************************/

void InitGlobals (GPtr globals)
{
	/* Globals get initialized at selectorParameters. */	
}

/*****************************************************************************/

/* Asks the user for the plug-in filter module's parameters. Note that
   the image size information is not yet defined at this point. Also, do
   not assume that the calling program will call this routine every time the
   filter is run (it may save the data held by the parameters handle in
   a macro file). Initialize any single-time info here. */

void DoParameters (GPtr globals)

{
	ValidateParameters (globals);
	
	gQueryForParameters = TRUE;
	/* If we're here, that means we're being called for the first time. */
}

/*****************************************************************************/

/* Check parameters and make sure their valid, or initialize them. */

void ValidateParameters (GPtr globals)
{
	
	if (!gStuff->parameters)
	{

		short loop = 0;

		gStuff->parameters = NewHandle ((long) sizeof (TParameters));

		if (!gStuff->parameters)
		{
			gResult = memFullErr;
			return;
		}
		
		SetColor (&gSourceColor, 0, 0, 0, 0);
		CopyColor (&gSourceColor, &gTargetColor);

		gColor = gColorize = 0;
		
	}
}

/*****************************************************************************/

/* Prepare to filter an image.	If the plug-in filter needs a large amount
   of buffer memory, this routine should set the bufferSpace field to the
   number of bytes required. Also check for */

void DoPrepare (GPtr globals)
	{
	
	gStuff->bufferSpace = 0;

	}

/*****************************************************************************/

/* Does all of the filtering. */

void DoStart (GPtr globals)
	{
	
	short oldRepFactor = 0;
	
	if (!WarnAdvanceStateAvailable ())
	{
		gResult = userCanceledErr; // exit gracefully
		goto done;
	}

	if (!WarnColorServicesAvailable())
	{
		gResult = userCanceledErr; // exit gracefully
		goto done;
	}
	
	ValidateParameters (globals);
	/* if stuff hasn't been initialized that we need, do it,
		then go check if we've got scripting commands to
		override our settings */
	ReadScriptParams (globals); // update our parameters with the scripting parameters, if available

	if (gQueryForParameters)
	{
		gQueryForParameters = FALSE;
		if (!PromptUserForInput (globals))
			goto done; // canceled
	}

	if (gResult != noErr)
		goto done;

	done:
	
	// if one screws with the outData with a proxy, reset that here
		
	PISetRect (&gStuff->inRect, 0, 0, 0, 0);
	PISetRect (&gStuff->outRect, 0, 0, 0, 0);
	PISetRect (&gStuff->maskRect, 0, 0, 0, 0);

	}

/*****************************************************************************/

/* Call routines to filter the area. */

void DoFilterRect (GPtr globals)
{

	// do actual filtering here

	PISetRect (&gStuff->inRect, 0, 0, 0, 0);
	PISetRect (&gStuff->outRect, 0, 0, 0, 0);
	PISetRect (&gStuff->maskRect, 0, 0, 0, 0);
	
}

/*****************************************************************************/

/* Given that we do all of the filtering during the start phase, the continue
   phase is negligible. */

void DoContinue (GPtr globals)
	{
	
	/* Force these to zero.  We never should actually get here. */

	PISetRect (&gStuff->inRect, 0, 0, 0, 0);
	PISetRect (&gStuff->outRect, 0, 0, 0, 0);
	PISetRect (&gStuff->maskRect, 0, 0, 0, 0);
	
	}

/*****************************************************************************/

/* Do any necessary clean-up. */

void DoFinish (GPtr globals)
{	
	WriteScriptParams (globals); // writes our parameters to the scripting system
}

/*******************************************************************/
void SetColor (void *color, int16 color1, int16 color2, int16 color3, int16 color4)
{
	int16	*p = (int16 *)color;
	
	p[0] = color1;
	p[1] = color2;
	p[2] = color3;
	p[3] = color4;
}

/*******************************************************************/
void CopyColor (void *color1, void *color2)
{
	short loop = 0;
	int16	*p = (int16 *)color1;
	int16	*q = (int16 *)color2;
	
	for (loop = 0; loop < 4; loop++)
	{
		q[loop] = p[loop];
	}
}

/*******************************************************************/
void PopulateColorServicesInfo (GPtr globals, ColorServicesInfo *csinfo)
{
	// now populate color services info with stuff
	csinfo->selector = plugIncolorServicesConvertColor;
	csinfo->sourceSpace = gColor;
	csinfo->resultSpace = gColorize;
	csinfo->reservedSourceSpaceInfo = NULL; // must be null
	csinfo->reservedResultSpaceInfo = NULL; // must be null
	CopyColor(&gSourceColor, &csinfo->colorComponents);
	// on input, source color; on output, result color
	csinfo->reserved = NULL; // must be null
	csinfo->selectorParameter.pickerPrompt = NULL; // not needed
	csinfo->infoSize = sizeof(*csinfo);
}

/*******************************************************************/
