/*
	File: ColorMunger.h

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	Utilities for ColorMunger example filter module.
*/

#ifndef __ColorMunger_H__
#define __ColorMunger_H__

#include "PIFilter.h"
#include "PIUtilities.h"
#include "PIProperties.h"

/*****************************************************************************/

#define typeColorName		'tyCN'
#define cnRGB				'cRGB'
#define cnHSB				'cHSB'
#define cnCMYK				'cmyK'
#define cnLab				'cLab'
#define cnGray				'graY'
#define cnHSL				'cHSL'
#define cnXYZ				'cXYZ'

/*****************************************************************************/

#define kMin				0
#define kMax				32767

#define kPrecision			6

#define cnArrayValues		cnRGB,	\
							cnHSB,	\
							cnCMYK,	\
							cnLab,	\
							cnGray,	\
							cnHSL,	\
							cnXYZ
	
enum
{ // color modes
	mRGB,
	mHSB,
	mCMYK,
	mLab,
	mGray,
	mHSL,
	mXYZ
};

enum
{
	dSourceRGB = 3,
	dSourceHSB,
	dSourceCMYK,
	dSourceLab,
	dSourceGray,
	dSourceHSL,
	dSourceXYZ,
	dSourcePicker
};
#define dSource1		dSourceRGB
#define dSourceEnd		dSourceXYZ
enum
{
	dESource1 = 11,
	dESource2,
	dESource3,
	dESource4
};
#define	dEditSource1	dESource1
#define dEditSourceEnd	dESource4
#define dSourceGroupItem	15

enum
{
	dTargetRGB = 17,
	dTargetHSB,
	dTargetCMYK,
	dTargetLab,
	dTargetGray,
	dTargetHSL,
	dTargetXYZ,
	dTargetPicker
};
#define	dTarget1		dTargetRGB
#define dTargetEnd		dTargetXYZ
enum
{
	dETarget1 = 25,
	dETarget2,
	dETarget3,
	dETarget4
};
#define dEditTarget1	dETarget1
#define dEditTargetEnd	dETarget4

#define dTargetGroupItem	29

/*****************************************************************************/

typedef struct TParameters
	{
	short	queryForParameters;
	short	color; // source color
	int16	sourceColor[4]; // source color space
	short	colorize; // target color
	int16	targetColor[4]; // target color space
	} TParameters, *PParameters, **HParameters;

#define gParams				((PParameters) *gStuff->parameters)
#define gQueryForParameters (gParams->queryForParameters)
#define gColor				(gParams->color)
#define gSourceColor		(gParams->sourceColor)
#define gColorize			(gParams->colorize)
#define gTargetColor		(gParams->targetColor)

/*****************************************************************************/

extern Handle hDllInstance;

typedef struct Globals
{
	short 	result;
	FilterRecord *filterParamBlock;
} Globals, *GPtr, **GHdl;
	
#define gResult 		(globals->result)
#define gStuff  		(globals->filterParamBlock)

/*****************************************************************************/

void DoAbout (GPtr globals);
short PromptUserForInput (GPtr globals);
void DoFilterRect (GPtr globals);
void ValidateParameters (GPtr globals);
Boolean ReadScriptParams (GPtr globals);
OSErr WriteScriptParams (GPtr globals);
void SetColor (void *color, int16 color1, int16 color2,
			   int16 color3, int16 color4);
void CopyColor (void *color1, void *color2);
void PopulateColorServicesInfo (GPtr globals, ColorServicesInfo *csinfo);
void AssignArray (void *array,
					DescriptorKeyID key1,
					DescriptorKeyID key2, 
					DescriptorKeyID key3, 
					DescriptorKeyID key4 );
DescriptorKeyID Enum2Type (DescriptorClassID thisClass, int16 thisEnum);
int16 Type2Enum (DescriptorClassID thisClass, DescriptorKeyID thisType);

/*****************************************************************************/

#endif /* __ColorMunger_H__ */
