/*
	File: ColorMunger.r

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	Rez source file for Propetizer example.
*/

/**********************************************************************************/
/* Defines needed in other rez files */

#define plugInName			"ColorMunger"
#define VersionString		"4.0"

/********************************************************************************/

#if Macintosh
#include "Types.r"
#include "SysTypes.r"
#include "PIGeneral.r"
#include "PIUtilities.r"
#include "DialogUtilities.r"
#endif

#if MSWindows
#include "PIGeneral.h"
#include "PIUtilities.r"
#include "WinDialogUtils.r"
#endif

#include "PIActions.h"

/********************************************************************************/

#define plugInComment 		"colorMunger example filter plug-in"
#define vendorName			"AdobeSDK"

#define ourSuiteID			'sdKB'
#define ourClassID			ourSuiteID
#define ourEventID			'clrM'

#define typeColorName		'tyCN'
#define cnRGB				'cRGB'
#define cnHSB				'cHSB'
#define cnCMYK				'cmyK'
#define cnLab				'cLab'
#define cnGray				'graY'
#define cnHSL				'cHSL'
#define cnXYZ				'cXYZ'

/********************************************************************************/

resource 'PiPL' (ResourceID, purgeable)
	{
		{
		Kind { Filter },
		Name { plugInName "..." },
		Category { "Utilities" },
		Version { (latestFilterVersion << 16) | latestFilterSubVersion },

		#if Macintosh
		Code68K { Filter, $$ID },
		CodePowerPC { 0, 0, "" },
		#endif

		#if MSWindows
		CodeWin32X86 { "ENTRYPOINT" },
		#endif
		
		HasTerminology { ourClassID, ourEventID, ResourceID, "" },
		/* classID, eventID, aete ID, uniqueString */
		
		SupportedModes
			{
			noBitmap, doesSupportGrayScale,
			noIndexedColor, doesSupportRGBColor,
			doesSupportCMYKColor, doesSupportHSLColor,
			doesSupportHSBColor, doesSupportMultichannel,
			doesSupportDuotone, doesSupportLABColor
			},

		EnableInfo { "in (PSHOP_ImageMode, GrayScaleMode, RGBMode,"
					     "CMYKMode, HSLMode, HSBMode, MultichannelMode,"
					     "DuotoneMode, LabMode)" },
					     
		FilterCaseInfo
			{
				{
				/* Flat data, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Flat data with selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
				
				/* Floating selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Editable transparency, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Editable transparency, with selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Preserved transparency, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Preserved transparency, with selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination
				}
			}
		}
	};

resource 'PiMI' (ResourceID, purgeable)
{
	latestFilterVersion,
	latestFilterSubVersion,
	0,
	supportsGrayScale +
	supportsRGBColor +
	supportsCMYKColor +
	supportsHSLColor +
	supportsHSBColor +
	supportsMultichannel +
	supportsDuotone +
	supportsLABColor,
	'    ', /* No required host */
	{},
};

/* About box resources */

resource StringResource (AboutID, "About Text", purgeable)
{
	plugInName "\n\n"
	"Version " VersionString " "
	"Release " ReleaseString "\n"
	"Copyright  1996, Adobe Systems Incorporated.\n"
	"All rights reserved.\n\n"
	"An example plug-in filter showing the color services suite in Adobe Photoshop."
};

/* Parameters dialog box */

resource 'DLOG' (ResourceID+1, "UI", purgeable)
{
	{102, 148, 286, 644},
	movableDBoxProc,
	visible,
	noGoAway,
	0x0,
	ResourceID+1,
	plugInName
};

resource 'DITL' (ResourceID+1, "UI", purgeable)
{
	{	/* array DITLarray: 30 elements */
		/* [1] */
		{4, 431, 24, 491},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{36, 431, 56, 491},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{36, 16, 52, 100},
		RadioButton {
			enabled,
			"RGB"
		},
		/* [4] */
		{52, 16, 68, 100},
		RadioButton {
			enabled,
			"HSB"
		},
		/* [5] */
		{68, 16, 84, 100},
		RadioButton {
			enabled,
			"CMYK"
		},
		/* [6] */
		{84, 16, 100, 100},
		RadioButton {
			enabled,
			"Lab"
		},
		/* [7] */
		{100, 16, 116, 100},
		RadioButton {
			enabled,
			"Gray"
		},
		/* [8] */
		{116, 16, 132, 100},
		RadioButton {
			enabled,
			"HSL"
		},
		/* [9] */
		{132, 16, 148, 100},
		RadioButton {
			enabled,
			"XYZ"
		},
		/* [10] */
		{148, 16, 164, 80},
		Button {
			enabled,
			"Picker"
		},
		/* [11] */
		{40, 109, 62, 194},
		EditText {
			enabled,
			""
		},
		/* [12] */
		{67, 109, 89, 194},
		EditText {
			enabled,
			""
		},
		/* [13] */
		{94, 109, 116, 194},
		EditText {
			enabled,
			""
		},
		/* [14] */
		{121, 109, 143, 194},
		EditText {
			enabled,
			""
		},
		/* [15] */
		{16, 12, 168, 208},
		UserItem {
			disabled
		},
		/* [16] */
		{8, 16, 28, 68},
		StaticText {
			disabled,
			"Source"
		},
		/* [17] */
		{36, 224, 52, 308},
		RadioButton {
			enabled,
			"RGB"
		},
		/* [18] */
		{52, 224, 68, 308},
		RadioButton {
			enabled,
			"HSB"
		},
		/* [19] */
		{68, 224, 84, 308},
		RadioButton {
			enabled,
			"CMYK"
		},
		/* [20] */
		{84, 224, 100, 308},
		RadioButton {
			enabled,
			"Lab"
		},
		/* [21] */
		{100, 224, 116, 308},
		RadioButton {
			enabled,
			"Gray"
		},
		/* [22] */
		{116, 224, 132, 308},
		RadioButton {
			enabled,
			"HSL"
		},
		/* [23] */
		{132, 224, 148, 308},
		RadioButton {
			enabled,
			"XYZ"
		},
		/* [24] */
		{148, 224, 164, 288},
		Button {
			enabled,
			"Picker"
		},
		/* [25] */
		{40, 312, 62, 397},
		StaticText {
			enabled,
			""
		},
		/* [26] */
		{67, 312, 89, 397},
		StaticText {
			enabled,
			""
		},
		/* [27] */
		{94, 312, 116, 397},
		StaticText {
			enabled,
			""
		},
		/* [28] */
		{121, 312, 143, 397},
		StaticText {
			enabled,
			""
		},
		/* [29] */
		{16, 216, 168, 412},
		UserItem {
			disabled
		},
		/* [30] */
		{8, 224, 28, 300},
		StaticText {
			disabled,
			"Convert to"
		}
	}
};

	
/* Scripting resource */

resource 'aete' (ResourceID, purgeable)
{
	1, 0, english, roman,									/* aete version and language specifiers */
	{
		vendorName,											/* vendor suite name */
		"Adobe example plug-ins",							/* optional description */
		ourSuiteID,											/* suite ID */
		1,													/* suite code, must be 1 */
		1,													/* suite level, must be 1 */
		{													/* structure for filters */
			vendorName " " plugInName,						/* unique filter name */
			plugInName " plug-in filter",					/* optional description */
			ourClassID,										/* class ID, must be unique or Suite ID */
			ourEventID,										/* event ID, must be unique */
			
			NO_REPLY,										/* never a reply */
			IMAGE_DIRECT_PARAMETER,							/* direct parameter, used by Photoshop */
			{												/* parameters here, if any */
				"color",									/* parameter name */
				keyColor,									/* parameter key ID */
				typeColorName,								/* parameter type ID */
				"source color",								/* optional description */
				flagsSingleParameter,						/* parameter flags */				

				"colorize",									/* parameter name */
				keyColorize,								/* parameter key ID */
				typeColorName,								/* parameter type ID */
				"target color",								/* optional description */
				flagsSingleParameter,						/* parameter flags */
				
				"foreground color",							// you get the idea
				keyForegroundColor,
				classColor,
				"initial color",
				flagsSingleParameter				
			}
		},
		{},													/* non-filter plug-in class here */
		{},													/* comparison ops (not supported) */
		{													/* any enumerations */
			typeColorName,									// type color name
			{
				"RGB",
				cnRGB,
				"Red-Green-Blue",
				
				"HSB",
				cnHSB,
				"Hue-Saturation-Brightness",
				
				"CMYK",
				cnCMYK,
				"Cyan-Magenta-Yellow-Black",
				
				"Lab",
				cnLab,
				"Luminance-A-B",
				
				"Gray",
				cnGray,
				"Grayscale",
				
				"HSL",
				cnHSL,
				"Hue-Saturation-Luminance",
				
				"XYZ",
				cnXYZ,
				"X-Y-Z"
			}
				
		}

	}
};