/*
	File: PropetizerScripting.c

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	C source file for scripting functions for Propetizer example.
*/

#include "ColorMunger.h"

/*****************************************************************************/
/* Checks the parameters against scripting-returned parameters, if any, and
   updates our parameters to match ones given to us by the scripting system. */

Boolean ReadScriptParams (GPtr globals)
{
	PIReadDescriptor			token = NULL, subtoken = NULL;
	PIDescriptorHandle			dh = NULL;
	DescriptorKeyID				key = 0;
	DescriptorTypeID			type = 0;
	DescriptorKeyIDArray		array = { keyColor, keyColorize, keyForegroundColor, NULLID };
	DescriptorKeyIDArray		subarray = { keyCyan, keyMagenta, keyYellow, keyBlack, NULLID };
	int32						flags = 0;
	Handle						h = NULL;
	OSErr						gotErr = noErr, stickyError = noErr;
	Boolean						returnValue = true;
	long						x = 0;
	
	if (DescriptorAvailable())
	{
		token = OpenReader(array);
		if (token)
		{
			while (PIGetKey(token, &key, &type, &flags))
			{
				switch (key)
				{
					case keyColor:
						PIGetEnum(token, &type);
						gColor = Type2Enum(classColor, type);
						break;
					case keyColorize:
						PIGetEnum(token, &type);
						gColorize = Type2Enum(classColor, type);
						break;
					case keyForegroundColor:
						PIGetObj(token, NULL, &dh); // we've already been handed the type @ GetKey
						if (dh != NULL)
						{
							switch (type)
							{
								case classRGBColor:
									AssignArray(&subarray, keyRed, keyGreen, keyBlue, NULLID);
									break;
								case classGrayscale:
									AssignArray(&subarray, keyGray, NULLID, NULLID, NULLID);
									break;
								case classHSBColor:
									AssignArray(&subarray, keyHue, keySaturation, keyBrightness, NULLID);
									break;
								case classLabColor:
									AssignArray(&subarray, keyLuminance, keyA, keyB, NULLID);
								case classUnspecifiedColor: // use for all other colors
								case classCMYKColor:
								default:
									AssignArray(&subarray, keyCyan, keyMagenta, keyYellow, keyBlack);
									break;
							}
							subtoken = OpenReadDesc(dh, subarray);
							if (subtoken)
							{ // we're about to get a color.
								SetColor(&gSourceColor, 0, 0, 0, 0);
								while (PIGetKey(subtoken, &key, &type, &flags))
								{
									switch (key)
									{
										case keyRed:
										case keyCyan:
										case keyGray:
										case keyHue:
										case keyLuminance:
											PIGetInt(subtoken, &x);
											gSourceColor[0] = (int16)x;
											break;
										case keyGreen:
										case keyMagenta:
										case keySaturation:
										case keyA:
											PIGetInt(subtoken, &x);
											gSourceColor[1] = (int16)x;
											break;
										case keyBlue:
										case keyYellow:
										case keyBrightness:
										case keyB:
											PIGetInt(subtoken, &x);
											gSourceColor[2] = (int16)x;
											break;
										case keyBlack:
											PIGetInt(subtoken, &x);
											gSourceColor[3] = (int16)x;
											break;
									}
								}
								stickyError = CloseReadDesc(subtoken);
								subtoken = NULL; // just in case
								PIDisposeHandle(dh);
								dh = NULL;
							
								if (stickyError)
								{
									if (stickyError == errMissingParameter)
										;
										/* (descriptorKeyIDArray != NULL) Missing a parameter.
						      				 Walk IDarray for which one. */
									else
										gResult = stickyError; // real error occurred

								}
							} // dh == NULL
						}
					}
			}
			stickyError = CloseReader(&token); // we're done, dispose.
			
			if (stickyError)
			{
				if (stickyError == errMissingParameter)
					;
					/* (descriptorKeyIDArray != NULL) Missing a parameter.
				       Walk IDarray for which one. */
				else
					gResult = stickyError; // real error occurred
			}
		}		
		gQueryForParameters = returnValue = PlayDialog();
		/* return TRUE if want to show our Dialog */			
	}
	return returnValue;
}

/*****************************************************************************/
/* Writes our parameters to the scripting system. */

OSErr WriteScriptParams (GPtr globals)
{
	PIWriteDescriptor			token = NULL, subtoken = NULL;
	OSErr						gotErr = noErr, stickyError = noErr;
	DescriptorKeyIDArray		array = { keyCyan, keyMagenta, keyYellow, keyBlack, NULLID };
	PIDescriptorHandle			h = NULL;
	DescriptorClassID			useClass = classUnspecifiedColor; // default class
	short						loop = 0;
	
	if (DescriptorAvailable())
	{
		token = OpenWriter();
		if (token)
		{					
			PIPutEnum(token, keyColor, typeColorName, Enum2Type(classColor, gColor));
			PIPutEnum(token, keyColorize, typeColorName, Enum2Type(classColor, gColorize));
			subtoken = OpenWriter();
			if (subtoken)
			{
				switch (gColor)
				{
					case mRGB:
						useClass = classRGBColor;
						AssignArray(&array, keyRed, keyGreen, keyBlue, NULLID);
						break;
					case mHSB:
						useClass = classHSBColor;
						AssignArray(&array, keyHue, keySaturation, keyBrightness, NULLID); 
						break;
					case mGray:
						useClass = classGrayscale;
						AssignArray(&array, keyGray, NULLID, NULLID, NULLID);
						break;
					case mLab:
						useClass = classLabColor;
						AssignArray(&array, keyLuminance, keyA, keyB, NULLID);
						break;
					case mCMYK:
						useClass = classCMYKColor;
						AssignArray(&array, keyCyan, keyMagenta, keyYellow, keyBlack);
						break;
					default:
						useClass = classUnspecifiedColor;
						AssignArray(&array, keyCyan, keyMagenta, keyYellow, keyBlack);
						break;
				}
				
				// now actually store values
				for (loop = 0; loop < 4; loop++)
				{
					if (array[loop] != NULLID)
						PIPutInt(subtoken, array[loop], gSourceColor[loop]);
				}
				
				stickyError = CloseWriteDesc(subtoken, &h);
				subtoken = NULL;

				if (stickyError == noErr && h != NULL)
				{
					PIPutObj(token, keyForegroundColor, useClass, &h);
					// routine disposes handle for us
				}
			}
		}
		gotErr = CloseWriter(&token); // closes and sets to dialogOptional
		/* done.  Now pass handle on to Photoshop */
	}

	return gotErr;
}

/*****************************************************************************/
/* Takes an enumeration and returns a type									 */

DescriptorKeyID Enum2Type (DescriptorClassID thisClass, short thisEnum)
{
	const DescriptorEnumID cnArray[] = { cnArrayValues };

	switch (thisClass)
	{
		case classColor:
			return cnArray[thisEnum];
			break;
	}
}

/*****************************************************************************/
/* Takes the GradientMode and returns the plug-in mode		  			     */

int16 Type2Enum (DescriptorClassID thisClass, DescriptorKeyID thisType)
{
	int16 loop;
	int16 retn = 0;
	
	const DescriptorEnumID cnArray[] = { cnArrayValues };

	switch (thisClass)
	{
		case classColor:
			for (loop = 0; loop < (sizeof(cnArray) / sizeof(PIType)); loop++)
				if (cnArray[loop] == thisType)
					retn = loop;
			break;
	}
	return retn;
}

/*****************************************************************************/
// Assigns a set of up to 4 values (and null) to an array
void AssignArray (void *array,
					DescriptorKeyID key1, 
					DescriptorKeyID key2,
					DescriptorKeyID key3,
					DescriptorKeyID key4 )
{
	DescriptorKeyID		*p = (DescriptorKeyID *)array;
	
	p[0] = key1;
	p[1] = key2;
	p[2] = key3;
	p[3] = key4;
	p[4] = NULLID;
}

/*****************************************************************************/

