/*
	File: DissolveSansUIWin.c

	Copyright 1991-6, Adobe Systems Incorporated.
	All rights reserved.

	C source file for Windows specific code for DissolveSans example.
*/

#include "DissolveSans.h"

/****************************************************************************/

void	ShowOutputBuffer(GPtr globals, HWND hPrev);
Boolean Validate (GPtr globals, const HWND hDlg, int *item);

/*****************************************************************************/

/* Calculates the rectangle we will want to use for the proxy for an image. */

void GetProxyItemRect (HWND	hDlg, Rect *proxyItemRect)
	{

	RECT	wRect;
	
	GetWindowRect(GetDlgItem(hDlg, kProxyItem), &wRect);	
	ScreenToClient (hDlg, (LPPOINT)&wRect);
	ScreenToClient (hDlg, (LPPOINT)&(wRect.right));

	proxyItemRect->left = (short)wRect.left;
	proxyItemRect->top  = (short)wRect.top;
	proxyItemRect->right = (short)wRect.right;
	proxyItemRect->bottom = (short)wRect.bottom;
	
	}

/*****************************************************************************/

void DoAbout (GPtr globals)
{
	ShowAbout((AboutRecordPtr)gStuff, hDllInstance, AboutID);
}

/*****************************************************************************/

void ShowOutputBuffer(GPtr globals, HWND hDlg)
{
	PSPixelMap pixels;
	PSPixelMask mask;
	PAINTSTRUCT  ps;
	Rect  itemBounds;
	RECT  wRect;
	POINT	mapOrigin; 
	HDC		hDC;

	GetWindowRect(GetDlgItem(hDlg, kProxyItem), &wRect);
	mapOrigin.x = 0;
	mapOrigin.y = 0;
	ClientToScreen(hDlg, &mapOrigin);

	itemBounds.left = (short)(((wRect.right + wRect.left - gStuff->outRect.right + gStuff->outRect.left) / 2) - mapOrigin.x);
	itemBounds.top = (short)(((wRect.bottom + wRect.top - gStuff->outRect.bottom + gStuff->outRect.top) / 2) - mapOrigin.y);
	itemBounds.right = (short)(itemBounds.left + (gStuff->outRect.right - gStuff->outRect.left));
	itemBounds.bottom = (short)(itemBounds.top + (gStuff->outRect.bottom - gStuff->outRect.top));
	
	hDC = BeginPaint(hDlg, &ps);	

	wRect.left = itemBounds.left;
	wRect.top = itemBounds.top;
	wRect.right = itemBounds.right;
	wRect.bottom = itemBounds.bottom;

	InflateRect(&wRect, kGutter, kGutter);
	//FillRect(hDC, &wRect, (HBRUSH) (COLOR_WINDOW));
	FrameRect(hDC, &wRect, GetStockObject(BLACK_BRUSH));	
	InflateRect(&wRect, -kGutter, -kGutter);
	
	pixels.version = 1;
	pixels.bounds.top = gStuff->outRect.top;
	pixels.bounds.left = gStuff->outRect.left;
	pixels.bounds.bottom = gStuff->outRect.bottom;
	pixels.bounds.right = gStuff->outRect.right;
	pixels.imageMode = gStuff->imageMode;
	pixels.rowBytes = gStuff->outRowBytes;
	pixels.colBytes = gStuff->outHiPlane - gStuff->outLoPlane + 1;
	pixels.planeBytes = 1;
	pixels.baseAddr = gStuff->outData;

	pixels.mat = NULL;
	pixels.masks = NULL;
	pixels.maskPhaseRow = 0;
	pixels.maskPhaseCol = 0;

	if (gStuff->isFloating) {
		mask.next = NULL;
		mask.maskData = gStuff->maskData;
		mask.rowBytes = gStuff->maskRowBytes;
		mask.colBytes = 1;
		mask.maskDescription = kSimplePSMask;
	
		pixels.masks = &mask;
	} else if ((gStuff->inLayerPlanes != 0) && (gStuff->inTransparencyMask != 0)) {
		mask.next = NULL;
		mask.maskData = ((char *) gStuff->outData) + (gStuff->outHiPlane - gStuff->outLoPlane);
		mask.rowBytes = gStuff->outRowBytes;
		mask.colBytes = gStuff->outHiPlane - gStuff->outLoPlane + 1;
		mask.maskDescription = kSimplePSMask;
	
		pixels.masks = &mask;
	}

	(gStuff->displayPixels)(&pixels, &pixels.bounds, itemBounds.top, itemBounds.left, (void *)hDC);

	EndPaint(hDlg, (LPPAINTSTRUCT) &ps);
}

/*****************************************************************************/
// Because we want to wait until the user changes the focus
// from an EditText field to something else, and because we
// want to ignore validation if the user Cancels, this routine
// does round of validating fields if it's been flagged.

Boolean Validate (GPtr globals, const HWND hDlg, int *item)
{
	int32	x = 0;
	Boolean retn = true;
	short	numberErr = noErr;
	RECT	imageRect;

	switch (*item)
	{
		case  kEditText: // user moving to something not the edit text
			numberErr = FetchNumber(hDlg, 
							   kEditText, 
							   kPercentMin, 
							   kPercentMax, 
							   &x);

			if (numberErr != noErr)
			{ // shows alert if there's an error
				AlertNumber(hDlg,
							kEditText,
							kPercentMin,
							kPercentMax,
							&x,
							hDllInstance,
							AlertID,
							numberErr);
				retn = false; // had to pop an error, stay here
			}

			gPercent = (short)x;

			// Do Filtering operation
			DispatchFilterRect (globals, false);

			// Invalidate Proxy Item
			GetWindowRect(GetDlgItem(hDlg, kProxyItem), &imageRect);
			ScreenToClient (hDlg, (LPPOINT)&imageRect);
			ScreenToClient (hDlg, (LPPOINT)&(imageRect.right));
			InvalidateRect (hDlg, &imageRect, FALSE);

			break;
	}
	*item = 0;
	return retn;
}

/*****************************************************************************/

BOOL WINAPI DissolveProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)       // Win32 Change
{
	static short	lastPercent = kPercentMin; /* called multiple times, */
	static BOOL		lastIgnoreSelection = FALSE; /* so these can only be */
	static OSType	lastDisposition = NULLID; /* inited once */
	RECT			imageRect;
	int				item, cmd;
	short			numberErr = 0;
	int32			x = 0;
	static int		lastItem = 0;
	static GPtr		globals = NULL;		  /* need to be static */

	switch  (wMsg)
	{

		case  WM_INITDIALOG:

			/* set up globals	*/

			globals = (GPtr) lParam;

			CenterDialog(hDlg);
			
			lastPercent = gPercent;
			lastDisposition = gDisposition;
			lastIgnoreSelection = gIgnoreSelection;

			if (gUseAdvance)
			{ // set up for proxy
				GetProxyItemRect (hDlg, &gProxyRect);
				SetupFilterRecordForProxy (globals);
				if (gResult == noErr)
				{ // everythings fine, do this
					DispatchFilterRect (globals, false);
				}
				else
				{ // advanceState barfed
					gUseAdvance = false;
					gResult = noErr; // reset
				}
			}

			StuffNumber(hDlg, kEditText,(int32)lastPercent);
			
			SetRadioGroupState(hDlg, kFirstRadio, kLastRadio,
				 kFirstRadio + GetDisposition(lastDisposition));
			
			SetCheckBoxState (hDlg, kEntireImage, lastIgnoreSelection);
			ShowHideItem(hDlg, kEntireImage, gStuff->haveMask);

			SelectTextItem(hDlg, kEditText);

			/* drop into PAINT */
		case WM_PAINT:
			if (gUseAdvance) ShowOutputBuffer(globals, hDlg);	 // Draw Proxy Item
			return FALSE;
			break;

		case  WM_COMMAND:
	  		item = COMMANDID (wParam);              // WIN32 Change
			cmd = HIWORD (wParam);

			switch  (item)
			{
				case kEditText:
					if (Validate(globals, hDlg, &lastItem))
					{
						if (cmd == EN_KILLFOCUS)
							lastItem = kEditText; // validate will get this
						else if (cmd == EN_CHANGE)
						{
							numberErr = FetchNumber(hDlg,
													kEditText,
													kPercentMin,
													kPercentMax,
													&x);
							if (numberErr == noErr && x != gPercent)
							{ // different number, update
								gPercent = (short)x;

								// Do Filtering operation
								DispatchFilterRect (globals, false);

								// Invalidate Proxy Item
								GetWindowRect(GetDlgItem(hDlg, kProxyItem), &imageRect);
								ScreenToClient (hDlg, (LPPOINT)&imageRect);
								ScreenToClient (hDlg, (LPPOINT)&(imageRect.right));
								InvalidateRect (hDlg, &imageRect, FALSE);
							}
						}
					}
					break;

				case kEntireImage:
					if (Validate(globals, hDlg, &lastItem))
					{
						if (cmd == BN_CLICKED)
						{	
							gIgnoreSelection = !gIgnoreSelection;
							SetCheckBoxState (hDlg, kEntireImage, gIgnoreSelection);
							
							// Do Filtering operation
							DispatchFilterRect (globals, false);
			
							// Invalidate Proxy Item
							GetWindowRect(GetDlgItem(hDlg, kProxyItem), &imageRect);
							ScreenToClient (hDlg, (LPPOINT)&imageRect);
							ScreenToClient (hDlg, (LPPOINT)&(imageRect.right));
							InvalidateRect (hDlg, &imageRect, FALSE);
							return FALSE;
						}
					}
					break;
				case ok:
					if (cmd == BN_CLICKED)
					{
						lastItem = kEditText; // force validate
						if (Validate(globals, hDlg, &lastItem))
						{
							EndDialog(hDlg, item); 
							return TRUE;
						}
					}
					else Validate(globals, hDlg, &lastItem);
						
					break;
				case cancel:
					if (cmd == BN_CLICKED)
					{
						gPercent = lastPercent;
						gDisposition = lastDisposition;
						gIgnoreSelection = lastIgnoreSelection;
						gResult = userCanceledErr;
						EndDialog(hDlg, item);          // WIN32 change
						return TRUE;
					}
					// don't ever validate during cancel
					break;
				default:
					if (item >= kFirstRadio && item <= kLastRadio) 
					{
						if (Validate(globals, hDlg, &lastItem))
						{
							if (cmd == BN_CLICKED)
							{
								SetRadioGroupState(hDlg, kFirstRadio, kLastRadio, wParam);		
								SetDisposition(globals, (short)(wParam - kFirstRadio));
						
								// Do filtering ops
								DispatchFilterRect (globals, false);
					
								// Invalidate Proxy Item
								GetWindowRect(GetDlgItem(hDlg, kProxyItem), &imageRect);
								ScreenToClient (hDlg, (LPPOINT)&imageRect);
								ScreenToClient (hDlg, (LPPOINT)&(imageRect.right));
								InvalidateRect (hDlg, &imageRect, FALSE);
								return FALSE;
							}
						}
					}
					break;
			} // switch (item)
			break; // case WM_COMMAND
		default:
			return FALSE;
			break;
		} // switch

	return  TRUE;
}

/*****************************************************************************/
short PromptUserForInput (GPtr globals)
{

	int    nResult;
	PlatformData *platform;

	platform = ((FilterRecordPtr) gStuff)->platformData;

	/* Query the user for percent dissolve. */

	nResult = DialogBoxParam(hDllInstance,
							 (LPSTR)"DISSOLVESANSPARAM",
							 (HWND)platform->hwnd,
							 (FARPROC)DissolveProc,
							 (LPARAM)globals);

	return (nResult == ok);

}	

/****************************************************************************/
/* Example for ShowAlert() function which takes a string ID as parameter    */
/* and displays a message box                                               */
/****************************************************************************/

short ShowAlert (short stringID)
{
	char szMessage[256];
	char szTitle[128];

	LoadString(hDllInstance, stringID, szMessage, sizeof szMessage);
	LoadString(hDllInstance, 2, szTitle, sizeof szTitle);
	return  ( MessageBox((HWND)NULL, (LPSTR)szMessage, (LPSTR)szTitle, MB_OK | MB_ICONHAND) );

}

/*****************************************************************************/

/* Initialization and termination code for window's dlls. */

/* ------------------------------------------------
 *   Code from Borland's window's dll example code.
 * ------------------------------------------------
 */
#if defined(__BORLANDC__)
// Turn off warning: Parameter '' is never used; effects next function only
#pragma argsused
#endif

#ifdef WIN32

// Every 32-Bit DLL has an entry point DLLInit

BOOL APIENTRY DLLInit(HANDLE hInstance, DWORD fdwReason, LPVOID lpReserved)
{

	if (fdwReason == DLL_PROCESS_ATTACH)
		hDllInstance = hInstance;

	return TRUE;   // Indicate that the DLL was initialized successfully.
}

#else
// Every 16-Bit DLL has an entry point LibMain and an exit point WEP.
int FAR PASCAL LibMain( HANDLE hInstance, WORD wDataSegment,
								   WORD wHeapSize, LPSTR lpszCmdLine )
{
	// Required when using Zortech; causes blink to include startup code
	extern __acrtused_dll;

	// The startup code for the DLL initializes the local heap (if there is one)
	// with a call to LocalInit which locks the data segment.
	if ( wHeapSize != 0 )
		UnlockData( 0 );

	hDllInstance = hInstance;
	return 1;   // Indicate that the DLL was initialized successfully.
}

int FAR PASCAL WEP(int nParam)
{
	switch  (nParam) {
	  case  WEP_SYSTEM_EXIT: // System shutdown in progress
	  case  WEP_FREE_DLL   : // DLL use count is 0
	  default :              // Undefined;  ignore
			return  1;
	}
}
#endif
/*****************************************************************************/


