/*
	File: DissolveWith.c

	Copyright (c) 1990, Thomas Knoll.
	Copyright (c) 1991-6, Adobe Systems Incorporated.
	All rights reserved.

	C source file for DissolveWith example.
*/

#if __MWERKS__
#include <SetupA4.h> // A4-globals
#include <A4Stuff.h> // A4-globals
#endif

#if defined(THINK_C) || defined(__MWERKS__)
#define ENTRYPOINT main
#endif

#include "DissolveWith.h"

Handle hDllInstance = NULL;

/*****************************************************************************/

void InitGlobals (GPtr globals);
void DoParameters (GPtr globals);
void DoPrepare (GPtr globals);
void DoStart (GPtr globals);
void DoContinue (GPtr globals);
void DoFinish (GPtr globals);

/*****************************************************************************/
#if MSWindows
	   void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result);
#else
pascal void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result);				
#endif

/*****************************************************************************/

/*----------------------------------------------------------------------------
 * Have to export our info for PPCGlue stub which needs a descriptor built
 * to be called from within a 68k code fragment.
 *----------------------------------------------------------------------------
 */

#ifdef __powerc						// For "fat" PowerMac plug-ins
enum
{
	uppEntryProcInfo = kPascalStackBased
		 | RESULT_SIZE(0)
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(FilterRecord *)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(long *)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(short *)))
};
#endif

#ifdef __powerc
#pragma export on
RoutineDescriptor main_PPC = BUILD_ROUTINE_DESCRIPTOR(uppEntryProcInfo, main);
#pragma export off
#endif

/*****************************************************************************/

/* All calls to the plug-in module come through this routine. It must be
   placed first in the resource. To achieve this, most development systems
   require that this be the first routine in the source. */

#if MSWindows
	   void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result)				
#else
pascal void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result)				
#endif
	{
	
	Globals globalValues;
	GPtr globals = &globalValues;
	
	#if __MWERKS__
	EnterCodeResource(); // A4-globals
	#endif
	
	if (!*data)
		{
		
		InitGlobals (globals);

		*data = (long) NewHandle (sizeof (Globals));
		
		if (!*data)
			{
			*result = memFullErr;
			return;
			}
			
		** (GHdl) *data = globalValues;
		
		}
		
	globalValues = ** (GHdl) *data;
		
	gStuff = filterParamBlock;
	gResult = noErr;
		
	switch (selector)
		{
		
		case filterSelectorAbout:
			DoAbout (globals);
			break;
			
		case filterSelectorParameters:
			DoParameters (globals);
			break;
			
		case filterSelectorPrepare:
			DoPrepare (globals);
			break;
		
		case filterSelectorStart:
			DoStart (globals);
			break;
		
		case filterSelectorContinue:
			DoContinue (globals);
			break;
		
		case filterSelectorFinish:
			DoFinish (globals);
			break;
			
		default:
			gResult = filterBadParameters;
		
		}
		
	*result = gResult;
	** (GHdl) *data = globalValues;

	#if __MWERKS__
	ExitCodeResource(); // A4-globals
	#endif
}

/*****************************************************************************/

void InitGlobals (GPtr globals)
{
	/* Globals get initialized at selectorParameters. */
}

/*****************************************************************************/

/* Asks the user for the plug-in filter module's parameters. Note that
   the image size information is not yet defined at this point. Also, do
   not assume that the calling program will call this routine every time the
   filter is run (it may save the data held by the parameters handle in
   a macro file). Initialize any single-time info here. */

void DoParameters (GPtr globals)

{
	ValidateParameters (globals);
	
	gQueryForParameters = TRUE;
	/* If we're here, that means we're being called for the first time. */
}

/*****************************************************************************/

/* Check parameters and make sure their valid, or initialize them. */

void ValidateParameters (GPtr globals)
{
	if (!gStuff->parameters)
	{

		gStuff->parameters = NewHandle ((long) sizeof (TParameters));

		if (!gStuff->parameters)
		{
			gResult = memFullErr;
			return;
		}
		gPercent = 50;
		gDisposition = dispositionCool;
		gIgnoreSelection = false;
		gUseAdvance = false;
		gRowSkip = 1;
		// Adobe Premiere 4.2 Win is a little wacky, so check for that right now
		gHostIsPremiereWin = IsWindows(HostIsPremiere());
	}
}

/*****************************************************************************/

/* Prepare to filter an image.	If the plug-in filter needs a large amount
   of buffer memory, this routine should set the bufferSpace field to the
   number of bytes required. */

void DoPrepare (GPtr globals)
{
	short 	rowWidth = 0;
	short 	total = 0;
	long	oneRow = 0;
	long	inOutRow = 0;
	long	inOutAndMask = 0;
	
	gStuff->bufferSpace = 0;

	// Check maxSpace to determine if we can process more than a row at a time
	
	ValidateParameters (globals);
	// check to make sure gRowSkip has been initialized BEFORE we use it!
	
	total = gStuff->filterRect.bottom - gStuff->filterRect.top;
	rowWidth = gStuff->filterRect.right - gStuff->filterRect.left;
	
	oneRow = rowWidth * (gStuff->planes); // one row of data and its planes
	inOutRow = oneRow * 2; // inData, outData
	inOutAndMask = inOutRow + rowWidth; // maskData is only one plane (alpha)
	while (((inOutAndMask * gRowSkip) < gStuff->maxSpace) && (gRowSkip < total))
		gRowSkip++;
		
	gStuff->maxSpace = gRowSkip * inOutAndMask; // all we need

}

/*****************************************************************************/

void DoStart (GPtr globals)
{		
	ValidateParameters (globals);
	/* if stuff hasn't been initialized that we need, do it,
		then go check if we've got scripting commands to
		override our settings */
	
	// can use advanceState or not?
	gUseAdvance = AdvanceStateAvailable () &&
				  DisplayPixelsAvailable () &&
				  !gHostIsPremiereWin;
				  // Adobe Premiere for Windows doesn't do AdvanceState right
				  
	ReadScriptParams (globals); // update our parameters with the scripting parameters, if available

	if (gQueryForParameters)
	{
		PromptUserForInput (globals);	// Show the UI
		gQueryForParameters = FALSE;
	}

	if (gResult != noErr)
		return;
			
	// Do the actual filtering operation on the original image
			
	if (gUseAdvance)
		StartWithAdvanceState(globals);
	else
		StartNoAdvanceState(globals);

	}
	
/******************************************************************************/
/* Does all of the filtering with no advance state and proxies. */

void StartNoAdvanceState (GPtr globals)
	{
	DoInitialRect(globals);
	}

/*****************************************************************************/

void StartWithAdvanceState (GPtr globals)
{
	
	int16 row;
	int32 total;

	// Do the actual filtering operation on the original image
	
	total = gStuff->filterRect.bottom - gStuff->filterRect.top;
	
	gStuff->inLoPlane = gStuff->outLoPlane = 0;
	gStuff->inHiPlane = gStuff->outHiPlane = gStuff->planes - 1;
	
	/* use this to set up alpha channels */
	#if useAlpha
	if (!gStuff->isFloating)
	{
		gStuff->wantsAbsolute = true;
		
		if (gStuff->inLayerPlanes !=0 && gStuff->inTransparencyMask != 0)
			gStuff->inHiPlane = gStuff->absLayerPlanes - 1 +
								gStuff->absTransparencyMask +
								gStuff->absLayerMasks +
								gStuff->absInvertedLayerMasks +
								gStuff->absNonLayerPlanes;
		
		if (gStuff->outLayerPlanes !=0 && gStuff->outTransparencyMask !=0)
			gStuff->outHiPlane = gStuff->outLayerPlanes - 1 +
								 gStuff->outTransparencyMask +
								 gStuff->outLayerMasks +
								 gStuff->outInvertedLayerMasks +
								 gStuff->outNonLayerPlanes;
	}
	#endif
	
			
	gStuff->inRect.left = gStuff->outRect.left = gStuff->maskRect.left = gStuff->filterRect.left;
	gStuff->inRect.right = gStuff->outRect.right = gStuff->maskRect.right = gStuff->filterRect.right;

	gStuff->inputRate = long2fixed (1L);
	gStuff->maskRate = long2fixed (1L);
	
	for (row = gStuff->filterRect.top; row < gStuff->filterRect.bottom; row += gRowSkip)
		{
		
		if (TestAbort () || gResult != noErr)
			{
			gResult = userCanceledErr;
			goto done;
			}
			
		gStuff->inRect.top = gStuff->outRect.top = gStuff->maskRect.top = row;
		gStuff->inRect.bottom = gStuff->outRect.bottom = gStuff->maskRect.bottom = row + gRowSkip;
		
		gResult = AdvanceState ();
		
		if (gResult != noErr)
			goto done;
		
		DoFilterRect (globals, true);
		
		}
	
	done:
	
	PISetRect (&gStuff->inRect, 0, 0, 0, 0);
	PISetRect (&gStuff->outRect, 0, 0, 0, 0);
	PISetRect (&gStuff->maskRect, 0, 0, 0, 0);

	}

/******************************************************/

void DispatchFilterRect (GPtr globals, const Boolean doProgress)
{
	if (gUseAdvance)
		DoFilterRect (globals, doProgress);
	else
		; // DoFilterRectNoAdvanceState (globals);
}


/*********************************************************************/

/* Filter the area. */

void DoFilterRect (GPtr globals, const Boolean doProgress)
{

	short i, j, loop;
	short plane, expectedPlanes = 0;
	const short columns = gStuff->outRect.right - gStuff->outRect.left;
	const short rows = gStuff->outRect.bottom - gStuff->outRect.top;
	const short total = gStuff->filterRect.bottom - gStuff->filterRect.top;
	short percent = gPercent;
	unsigned8 color[4] = { 0, 0, 0, 0 };
	PIType disposition = gDisposition;
	
	unsigned8 *srcPtr = (unsigned8 *) gStuff->inData;

	unsigned8 *dstPtr = (unsigned8 *) gStuff->outData;
	
	unsigned8 *mskPtr = (unsigned8 *) gStuff->maskData;
	
	Boolean dissolve = false;
	Boolean maskPixel = false;
	
	// Convert color now so we don't have to do it later
	
	switch (disposition)

	{
		case dispositionClear:
			expectedPlanes = CSPlanesFromMode(gStuff->imageMode, expectedPlanes);
			for (loop = 0; loop < expectedPlanes; loop++)
				color[loop] = (unsigned8)gStuff->backColor[loop];
			break;
		case dispositionHot:
			SetColor (globals, &color[0], 255, 0, 0, 0, &expectedPlanes);
			break;
		case dispositionSick:
			SetColor (globals, &color[0], 0, 255, 0, 0, &expectedPlanes);
			break;
		case dispositionCool:
		default:
			SetColor (globals, &color[0], 0, 0, 255, 0, &expectedPlanes);
			break;
	}

	for (i=0; i < rows; i++)
		{

		srcPtr = (unsigned8 *) gStuff->inData + (i * gStuff->inRowBytes);
		dstPtr = (unsigned8 *) gStuff->outData + (i * gStuff->outRowBytes);
		mskPtr = (unsigned8 *) gStuff->maskData + (i * gStuff->maskRowBytes);	
			 
		if (doProgress)
			UpdateProgress ((long) (gStuff->outRect.top + i) - gStuff->filterRect.top,
							(long) total);

		for (j=0; j < columns; j++)
		{
		
		
				if (TestAbort ())
				{
					gResult = userCanceledErr;
					return;
				}			
		
			dissolve = ((unsigned16) Random ()) % 100 < percent;
		
			if (gStuff->haveMask)
				maskPixel = (!mskPtr [0]);
				
			if (!dissolve || (!gIgnoreSelection && maskPixel))
				for (plane = 0; plane < gStuff->planes; ++plane)
					dstPtr [plane] = srcPtr [plane];
			else
			{
				for (plane = gStuff->planes - 1; plane >= expectedPlanes; --plane)
					dstPtr [plane] = 255;
			
				if (gHostIsPremiereWin)
				{ // Premiere Windows reverses the planes, so we have to store
				  // these backwards
				  for ( ; plane >=0; plane--)
				  	dstPtr [plane] = color [(expectedPlanes-1)-plane];
				}
				else // do it normally
					for ( ; plane >= 0; plane--)
						dstPtr [plane] = color [plane];
			
			}
				
			srcPtr += gStuff->inHiPlane - gStuff->inLoPlane + 1;
			dstPtr += gStuff->planes;
			mskPtr++; // one byte per pixel;

		}	
	}

}

/*****************************************************************************/
/* Sets a pixel to a certain color */

void SetColor (GPtr globals,
				unsigned8 *dstPtr,
				unsigned8 c0,
				unsigned8 c1,
				unsigned8 c2,
				unsigned8 c3,
				int16 *planes)
{
	int16	color[4] = { 0, 0, 0, 0 };
	short	loop;
	int16	resultSpace = CSModeToSpace(gStuff->imageMode);
	
	// Set our array from the literal colors handed us
	CSSetColor(color, (int16)c0, (int16)c1, (int16)c2, (int16)c3);
	
	if (resultSpace != plugIncolorServicesChosenSpace)
	{ // we were able to map mode to sourceSpace.  Convert color.
	
		// Now convert from RGB to the current image mode
		CSConvertColor (plugIncolorServicesRGBSpace, resultSpace, &color[0]);
		
		// Tell me how many color planes are expected in this mode
		*planes = CSPlanesFromMode (gStuff->imageMode, *planes);
	}
	
	// Populate our planes with the returned color, and don't exceed the expected
	// number of planes for this color or we'll get a bounds error
	for (loop = 0; loop < *planes; loop++)
		dstPtr[loop] = (unsigned8)color[loop];
}

/*****************************************************************************/

/* Requests first part of the image to be filtered. */

void DoInitialRect (GPtr globals)
{
	int32 total;

	// Do the actual filtering operation on the original image
	
	total = gStuff->filterRect.bottom - gStuff->filterRect.top;
	
	gStuff->inLoPlane = gStuff->outLoPlane = 0;
	gStuff->inHiPlane = gStuff->outHiPlane = gStuff->planes - 1;
	
	/* use this to set up alpha channels */
	#if useAlpha
	if (!gStuff->isFloating)
	{
		gStuff->wantsAbsolute = true;
		
		if (gStuff->inLayerPlanes !=0 && gStuff->inTransparencyMask != 0)
			gStuff->inHiPlane = gStuff->absLayerPlanes - 1 +
								gStuff->absTransparencyMask +
								gStuff->absLayerMasks +
								gStuff->absInvertedLayerMasks +
								gStuff->absNonLayerPlanes;
		
		if (gStuff->outLayerPlanes !=0 && gStuff->outTransparencyMask !=0)
			gStuff->outHiPlane = gStuff->outLayerPlanes - 1 +
								 gStuff->outTransparencyMask +
								 gStuff->outLayerMasks +
								 gStuff->outInvertedLayerMasks +
								 gStuff->outNonLayerPlanes;
	}
	#endif

	gStuff->inputRate = long2fixed (1L);
	gStuff->maskRate = long2fixed (1L);
	
	gStuff->maskPadding = gStuff->outputPadding = gStuff->inputPadding = 255;
	
	gStuff->inRect = gStuff->outRect = gStuff->maskRect = gStuff->filterRect;
	gStuff->inRect.bottom = gStuff->outRect.bottom = gStuff->maskRect.bottom = gStuff->inRect.top + gRowSkip;

}

/*****************************************************************************/

/* If we do all of the filtering during the start phase, the continue
   phase is negligible. */

/* Otherwise (no AdvanceState) Filters the area and requests the next area. */

void DoContinue (GPtr globals)
{
	
	if (TestAbort () || gResult != noErr)
	{
		gResult = userCanceledErr;
		return;
	}
		
	if (!gUseAdvance)
	{
		DoFilterRect (globals, true); // NoAdvanceState (globals);

		if (!DoNextRect (globals))
			{
				PISetRect (&gStuff->inRect, 0, 0, 0, 0);
				PISetRect (&gStuff->outRect, 0, 0, 0, 0);
				PISetRect (&gStuff->maskRect, 0, 0, 0, 0);
			}
	}
	else
	{
		PISetRect (&gStuff->inRect, 0, 0, 0, 0);
		PISetRect (&gStuff->outRect, 0, 0, 0, 0);
		PISetRect (&gStuff->maskRect, 0, 0, 0, 0);
	}
}

/*****************************************************************************/

/* Request the next area. */

Boolean DoNextRect (GPtr globals)
	{

	gStuff->inRect.top += gRowSkip; // next batch
	gStuff->inRect.bottom += gRowSkip; // next batch
	gStuff->outRect = gStuff->maskRect = gStuff->inRect;
	
	return gStuff->inRect.top < gStuff->filterRect.bottom;

	}
	
/*****************************************************************************/

/* Do any necessary clean-up. */

void DoFinish (GPtr globals)
{
	WriteScriptParams (globals); // writes our parameters to the scripting system
}

/*****************************************************************************/

void scaleRect(Rect *l, short n, short d) {
	l->left = (l->left * n) / d;
	l->top = (l->top * n) / d;
	l->right = (l->right * n) / d;
	l->bottom = (l->bottom * n) / d;
}

void shrinkRect(Rect *l, short x, short y) {
	l->left += x;
	l->top += y;
	l->right -= x;
	l->bottom -= y;
}

void copyRect(Rect *l, Rect *r) {
	l->left = r->left;
	l->top = r->top;
	l->right = r->right;
	l->bottom = r->bottom;
}

/*****************************************************************************/

void SetupFilterRecordForProxy (GPtr globals)

	{

	short 	scaleFactor;

	CalcProxyScaleFactor (&gStuff->filterRect, &gProxyRect, &scaleFactor);	

	gStuff->inRect = gStuff->filterRect;
	
	scaleRect (&gStuff->inRect, 1, scaleFactor);
	
	gStuff->maskRect = gStuff->outRect = gStuff->inRect;

	gStuff->inLoPlane = gStuff->outLoPlane = 0;
	gStuff->inHiPlane = gStuff->outHiPlane = gStuff->planes - 1;

	gStuff->inputRate = long2fixed(scaleFactor);
	gStuff->maskRate = long2fixed(scaleFactor);
 
	gStuff->inputPadding = 255;
	gStuff->outputPadding = gStuff->inputPadding;
	gStuff->maskPadding = gStuff->inputPadding;
	
	gResult = AdvanceState ();

	// if gResult != noErr then AdvanceState barfed.  Pass that on.  Otherwise:
	// Now gStuff->inData should have the subsampled version of input data
	// and gStuff->outData should be a buffer the size of input data
	// to use as a temporary buffer to do intermediate filtering
	// to display proxies

	}

/***********************************************************************************/

/* Computes the scaled down rectangle and the scale factor for the proxy */

void CalcProxyScaleFactor (const Rect *filterRect, Rect *proxyRect, short *scaleFactor)
	{

	short filterHeight;
	short filterWidth;
	short itemHeight;
	short itemWidth;
	Point fraction;

	shrinkRect (proxyRect, kGutter, kGutter); 		 // for the border

	filterHeight = filterRect->bottom - filterRect->top;
	filterWidth  = filterRect->right  - filterRect->left;
	
	itemHeight = proxyRect->bottom - proxyRect->top;
	itemWidth  = proxyRect->right  - proxyRect->left;
	
	if (itemHeight > filterHeight)
		itemHeight = filterHeight;
		
	if (itemWidth > filterWidth)
		itemWidth = filterWidth;
	
	fraction.h = (short)((filterWidth + itemWidth) / itemWidth);
	fraction.v = (short)((filterHeight + itemHeight) / itemHeight);

	if (fraction.h > fraction.v) 
		*scaleFactor = fraction.h;
	else
		*scaleFactor = fraction.v;

	copyRect (proxyRect, filterRect);	
	scaleRect (proxyRect, 1, *scaleFactor);
	}

/****************************************************************************/