/*
	File: DissolveWith.r

	Copyright 1990-1, Thomas Knoll.
	Copyright 1992-6, Adobe Systems Incorporated.
	All rights reserved.

	Rez source file for Dissolve example.
*/

/************************************************************/

#define plugInName			"Dissolve-with-AppleScript"
#define VersionString		"4.0"

/************************************************************/

#if Macintosh
#include "Types.r"
#include "SysTypes.r"
#include "PIGeneral.r"
#include "PIUtilities.r"
#include "DialogUtilities.r"
#endif

#if MSWindows
#include "PIGeneral.h"
#include "PIUtilities.r"
#include "WinDialogUtils.r"
#endif

#include "PIActions.h"

/********************************************************************************/

#define plugInComment 		"dissolve example filter plug-in"
#define vendorName			"AdobeSDK"
#define keyDisposition 		'disP'
#define typeMood			'mooD'
#define dispositionClear	'moD0'
#define dispositionCool		'moD1'
#define dispositionHot		'moD2'
#define dispositionSick		'moD3'
#define ourSuiteID			'sdK2'
#define ourClassID			ourSuiteID
#define ourEventID			'disS'

/************************************************************/

resource 'PiPL' (ResourceID, purgeable)
	{
		{
		Kind { Filter },
		Name { plugInName "..." },
		Category { "Noise" },
		Version { (latestFilterVersion << 16) | latestFilterSubVersion },

		#if Macintosh
		Code68K { Filter, $$ID },
		CodePowerPC { 0, 0, "" },
		#endif

		#if MSWindows
		CodeWin32X86 { "ENTRYPOINT" },
		#endif
		
		HasTerminology { ourClassID, ourEventID, ResourceID, "" },
		/* classID, eventID, aete ID, uniqueString */
		
		SupportedModes
			{
			noBitmap, doesSupportGrayScale,
			noIndexedColor, doesSupportRGBColor,
			doesSupportCMYKColor, doesSupportHSLColor,
			doesSupportHSBColor, doesSupportMultichannel,
			doesSupportDuotone, doesSupportLABColor
			},
		
		EnableInfo { "in (PSHOP_ImageMode, GrayScaleMode, RGBMode,"
						 "CMYKMode, HSLMode, HSBMode, MultichannelMode,"
						 "DuotoneMode, LABMode)" },
	
		FilterCaseInfo
			{
				{
				/* Flat data, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				doNotCopySourceToDestination,
					
				/* Flat data with selection */
				inWhiteMat, outStraightData,
				writeOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				doNotCopySourceToDestination,
				
				/* Floating selection */
				inWhiteMat, outStraightData,
				writeOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				doNotCopySourceToDestination,
					
				/* Editable transparency, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				doNotCopySourceToDestination,
					
				/* Editable transparency, with selection */
				inWhiteMat, outStraightData,
				writeOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				doNotCopySourceToDestination,
					
				/* Preserved transparency, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				doNotCopySourceToDestination,
					
				/* Preserved transparency, with selection */
				inWhiteMat, outStraightData,
				writeOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				doNotCopySourceToDestination
				}
			}	
		}
	};

resource 'PiMI' (ResourceID, purgeable)
	{ latestFilterVersion,
	  latestFilterSubVersion,
	  0,
	  supportsGrayScale +
	  	supportsRGBColor +
		supportsCMYKColor +
		supportsHSLColor +
		supportsHSBColor +
		supportsMultichannel +
		supportsDuotone +
		supportsLABColor,
	  '    ', /* No required host */
	  {},
	 };

/* About resources */

resource StringResource (AboutID, "About Text", purgeable)
{
	plugInName "\n\n"
	"Version " VersionString " "
	"Release " ReleaseString "\n"
	"Copyright  1992-6, Adobe Systems Incorporated.\n"
	"All rights reserved.\n\n"
	"An example plug-in filter module for Adobe Photoshop."
};

/* Parameters dialog box */

resource 'DLOG' (ResourceID+1, "UI", purgeable)
{
	{96, 148, 208, 444},
	movableDBoxProc,
	visible,
	noGoAway,
	0x0,
	ResourceID+1,
	plugInName
};

resource 'DITL' (ResourceID+1, "UI", purgeable) {
	{	/* array DITLarray: 11 elements */
		/* [1] */
		{8, 228, 28, 288},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{36, 228, 56, 288},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{8, 161, 24, 190},
		EditText {
			enabled,
			""
		},
		/* [4] */
		{8, 77, 24, 153},
		StaticText {
			disabled,
			"Amount %:"
		},
		/* [5] */
		{4, 8, 64, 68},
		UserItem {
			disabled
		},
		/* [6] */
		{40, 72, 56, 153},
		StaticText {
			disabled,
			"Disposition:"
		},
		/* [7] */
		{40, 156, 56, 214},
		RadioButton {
			enabled,
			"Clear"
		},
		/* [8] */
		{56, 156, 72, 214},
		RadioButton {
			enabled,
			"Cool"
		},
		/* [9] */
		{72, 156, 88, 214},
		RadioButton {
			enabled,
			"Hot"
		},
		/* [10] */
		{88, 156, 104, 214},
		RadioButton {
			enabled,
			"Sick"
		},
		/* [11] */
		{88, 8, 104, 112},
		CheckBox {
			enabled,
			"Entire image"
		}
	}
};

/* Scripting resource */

resource 'aete' (ResourceID, purgeable)
{
	1, 0, english, roman,									/* aete version and language specifiers */
	{
		vendorName,											/* vendor suite name */
		"Adobe example plug-ins",							/* optional description */
		ourSuiteID,											/* suite ID */
		1,													/* suite code, must be 1 */
		1,													/* suite level, must be 1 */
		{													/* structure for filters */
			vendorName " dissolve",							/* unique filter name */
			"dissolve noise generator plug-in filter",		/* optional description */
			ourClassID,										/* class ID, must be unique or Suite ID */
			ourEventID,										/* event ID, must be unique */
			
			NO_REPLY,										/* never a reply */
			IMAGE_DIRECT_PARAMETER,							/* direct parameter, used by Photoshop */
			{												/* parameters here, if any */
				"amount",									/* parameter name */
				keyAmount,									/* parameter key ID */
				typeFloat,									/* parameter type ID */
				"dissolve amount",							/* optional description */
				flagsSingleParameter,						/* parameter flags */
				
				"AdobeSDK disposition",						// second parameter
				keyDisposition,								// parameter key ID
				typeMood,									// parameter type ID
				"dissolve disposition",						// optional description
				flagsEnumeratedParameter					// parameter flags

			}
		},
		{													/* non-filter plug-in class here */
		},
		{													/* comparison ops (not supported) */
		},
		{													/* any enumerations */
			typeMood,										/* type disposition 'mooD' */
			{
				vendorName " clear",						/* first value */
				dispositionClear,							/* 'moD0' */
				"clear headed",								/* optional description */
				
				vendorName " cool",							/* second value */
				dispositionCool,							/* 'moo1' */
				"got the blues",							/* optional description */
				
				vendorName " hot",							/* third value */
				dispositionHot,								/* 'moo2' */
				"red-faced",								/* optional description */
				
				vendorName " sick",							/* fourth value */
				dispositionSick,							/* 'moo3' */
				"green with envy"							/* optional description */
			} // actual names must be unique because we're up against all dictionaries
				
		}
	}
};