/*
	File: DissolveWithScripting.c

	Copyright (c) 1990, Thomas Knoll.
	Copyright (c) 1991-6, Adobe Systems Incorporated
	All rights reserved.

	C source file with scripting functions for DissolveWith example.
*/

#include "DissolveWith.h"

/*****************************************************************************/
/* Checks the parameters against scripting-returned parameters, if any, and
   updates our parameters to match ones given to us by the scripting system. */

Boolean ReadScriptParams (GPtr globals)
{
	double						x = 0;
	const double				minValue = kPercentMin, maxValue = kPercentMax;
	unsigned long				percentUnitPass = unitPercent;
	PIReadDescriptor			token = NULL;
	DescriptorKeyID				key = NULLID;
	DescriptorTypeID			type = NULLID;
	DescriptorKeyIDArray		array = { keyAmount, keyDisposition, NULLID };
	int32						flags = 0;
	OSErr						stickyError = noErr;
	Boolean						returnValue = true;
	
	if (DescriptorAvailable())
	{
		token = OpenReader(array);
		if (token)
		{
			while (PIGetKey(token, &key, &type, &flags))
			{
				switch (key)
				{
					case keyAmount:
						PIGetPinUnitFloat(token, &minValue, &maxValue, &percentUnitPass, &x);
						gPercent = (short)x;
						break;
					case keyDisposition:
						PIGetEnum(token, &gDisposition);
						break;
					case keyEntireImage:
						PIGetBool(token, &gIgnoreSelection);
						break;
					// ignore all other cases and classes
				}
			}

			stickyError = CloseReader(&token); // we're done, dispose.

			if (stickyError)
			{
				if (stickyError == errMissingParameter) // missedParamErr == -1715
					;
					/* (descriptorKeyIDArray != NULL)
					   missing parameter somewhere.  Walk IDarray to find which one. */
				else
					gResult = stickyError;
			}				
		}
		gQueryForParameters = returnValue = PlayDialog();
		/* return TRUE if want to show our Dialog */
	}
	return returnValue;
}
		
/*****************************************************************************/

/* Writes our parameters to the scripting system. */

OSErr WriteScriptParams (GPtr globals)
{
	double						percent = gPercent;
	PIDescriptorHandle			writeDesc = nil;
	PIWriteDescriptor			token = nil;
	OSErr						gotErr = noErr;
			
	if (DescriptorAvailable())
	{
		token = OpenWriter();
		if (token)
		{
			PIPutUnitFloat(token, keyAmount, unitPercent, &percent);
			PIPutEnum(token, keyDisposition, typeMood, gDisposition);
			if (gIgnoreSelection) PIPutBool(token, keyEntireImage, gIgnoreSelection);
			gotErr = CloseWriter(&token); // closes, sets recordInfo to dialogOptional
		/* done.  Now pass handle on to Photoshop */
		}
	}
	return gotErr;
}

/*****************************************************************************/
/* Takes the disposition and returns a corresponding enumerated number */

short GetDisposition (OSType disposition)

{
	switch (disposition)
	{
		case dispositionClear:
			return dialogDispositionClear;
			break;
		case dispositionHot:
			return dialogDispositionHot;
			break;
		case dispositionSick:
			return dialogDispositionSick;
			break;
		case dispositionCool:
		default:
			return dialogDispositionCool;
			break;
	}
}

/*****************************************************************************/
/* Takes the disposition enumeration and sets the corresponding disposition */

void SetDisposition (GPtr globals, short dispositionEnum)

{

	OSType dispositionType = dispositionCool;
	
	switch (dispositionEnum)
	{
		case dialogDispositionClear:
			dispositionType = dispositionClear;
			break;
		case dialogDispositionCool:
			dispositionType = dispositionCool;
			break;
		case dialogDispositionHot:
			dispositionType = dispositionHot;
			break;
		case dialogDispositionSick:
			dispositionType = dispositionSick;
			break;
	}
	
	gDisposition = dispositionType;
}

/*****************************************************************************/