/*
	File: Propetizer.c

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	C source file for Propetizer example.
*/

#if __MWERKS__
#include <SetupA4.h> // A4-globals
#include <A4Stuff.h> // A4-globals
#endif

#if defined(THINK_C) || defined(__MWERKS__)
#define ENTRYPOINT main
#endif

#include "Propetizer.h"

Handle hDllInstance = NULL;

/*****************************************************************************/

void InitGlobals (GPtr globals);
void DoParameters (GPtr globals);
void DoPrepare (GPtr globals);
void DoStart (GPtr globals);
void DoContinue (GPtr globals);
void DoFinish (GPtr globals);
void ZoomInRect (GPtr globals);
void ZoomOutRect (GPtr globals);
static void ApplyChannel (GPtr globals,
						  ReadChannelDesc *source,
						  PixelMemoryDesc *sDesc,
						  ReadChannelDesc *mask, 
						  PixelMemoryDesc *mDesc,
						  WriteChannelDesc *dest, 
						  ChannelReadPort destRead,
						  PixelMemoryDesc *dDesc,
						  PixelMemoryDesc *rDesc,
						  int32 *done,
						  int32 total);
static int32 AccountChannel (ReadChannelDesc *source,
									  ReadChannelDesc *mask,
									  WriteChannelDesc *dest);
static Boolean EqualVRects (VRect *a, VRect *b);
static void TrimVRect (VRect *dst, VRect *src);

/*****************************************************************************/

/* All calls to the plug-in module come through this routine. It must be
   placed first in the resource. To achieve this, most development systems
   require that this be the first routine in the source. */

#if MSWindows
	   void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result)				
#else
pascal void ENTRYPOINT (short selector,
						FilterRecord *filterParamBlock,
						long *data,
						short *result)				
#endif
	{
	
	Globals globalValues;
	GPtr globals = &globalValues;
	
	#if __MWERKS__
	EnterCodeResource(); // A4-globals
	#endif
	
	if (!*data)
		{
		
		InitGlobals (globals);

		*data = (long) NewHandle (sizeof (Globals));
		
		if (!*data)
			{
			*result = memFullErr;
			return;
			}
			
		** (GHdl) *data = globalValues;
		
		}
		
	globalValues = ** (GHdl) *data;
		
	gStuff = filterParamBlock;
	gResult = noErr;
		
	switch (selector)
		{
		
		case filterSelectorAbout:
			DoAbout (globals);
			break;
			
		case filterSelectorParameters:
			DoParameters (globals);
			break;
			
		case filterSelectorPrepare:
			DoPrepare (globals);
			break;
		
		case filterSelectorStart:
			DoStart (globals);
			break;
		
		case filterSelectorContinue:
			DoContinue (globals);
			break;
		
		case filterSelectorFinish:
			DoFinish (globals);
			break;
			
		default:
			gResult = filterBadParameters;
		
		}
		
	*result = gResult;
	** (GHdl) *data = globalValues;

	#if __MWERKS__
	ExitCodeResource(); // A4-globals
	#endif
}

/*****************************************************************************/

void InitGlobals (GPtr globals)
{
	/* Globals get initialized at selectorParameters. */	
}

/*****************************************************************************/

/* Asks the user for the plug-in filter module's parameters. Note that
   the image size information is not yet defined at this point. Also, do
   not assume that the calling program will call this routine every time the
   filter is run (it may save the data held by the parameters handle in
   a macro file). Initialize any single-time info here. */

void DoParameters (GPtr globals)

{
	ValidateParameters (globals);
	
	gQueryForParameters = TRUE;
	/* If we're here, that means we're being called for the first time. */
}

/*****************************************************************************/

/* Check parameters and make sure their valid, or initialize them. */

void ValidateParameters (GPtr globals)
{
	if (!gStuff->parameters)
	{

		gStuff->parameters = NewHandle ((long) sizeof (TParameters));

		if (!gStuff->parameters)
		{
			gResult = memFullErr;
			return;
		}
		PropertiesToGlobals(globals); // sets up globals

	}
	gRepFactor = (float)1.0;
	gZoomFactor = kNumZooms; // always start at this

}

/*****************************************************************************/

/* Prepare to filter an image.	If the plug-in filter needs a large amount
   of buffer memory, this routine should set the bufferSpace field to the
   number of bytes required. Also check for */

void DoPrepare (GPtr globals)
	{
	
	gStuff->bufferSpace = 0;

	}

/*****************************************************************************/

/* Does all of the filtering. */

void DoStart (GPtr globals)
	{
	
	float oldRepFactor;
	
	if (!WarnAdvanceStateAvailable ())
	{
		gResult = userCanceledErr; // exit gracefully
		goto done;
	}

	if (!WarnDisplayPixelsAvailable())
	{
		gResult = userCanceledErr; // exit gracefully
		goto done;
	}
	
	ValidateParameters (globals);
	/* if stuff hasn't been initialized that we need, do it,
		then go check if we've got scripting commands to
		override our settings */
	ReadScriptParams (globals); // update our parameters with the scripting parameters, if available

	if (gQueryForParameters)
	{
		gQueryForParameters = FALSE;
		if (!PromptUserForInput (globals))
			goto done; // canceled
	}

	if (gResult != noErr)
		goto done;
	
	// Update properties based on UI/scripting
	GlobalsToProperties(globals);
	
	// Do the actual filtering operation on the original image here
	
	AddComment (globals);
	
	done:
	
	oldRepFactor = gRepFactor; // store temporarily
	gRepFactor = (float)1.0;
	DoZoomRect(globals); // reset outRect
	gRepFactor = oldRepFactor; // reset
	
	PISetRect (&gStuff->inRect, 0, 0, 0, 0);
	PISetRect (&gStuff->outRect, 0, 0, 0, 0);
	PISetRect (&gStuff->maskRect, 0, 0, 0, 0);

	}

/*****************************************************************************/

/* Call routines to filter the area. */

void DoFilterRect (GPtr globals)
{

	// do actual filtering here

	PISetRect (&gStuff->inRect, 0, 0, 0, 0);
	PISetRect (&gStuff->outRect, 0, 0, 0, 0);
	PISetRect (&gStuff->maskRect, 0, 0, 0, 0);
	
}

/*****************************************************************************/

/* Given that we do all of the filtering during the start phase, the continue
   phase is negligible. */

void DoContinue (GPtr globals)
	{
	
	/* Force these to zero.  We never should actually get here. */

	PISetRect (&gStuff->inRect, 0, 0, 0, 0);
	PISetRect (&gStuff->outRect, 0, 0, 0, 0);
	PISetRect (&gStuff->maskRect, 0, 0, 0, 0);
	
	}

/*****************************************************************************/

/* Do any necessary clean-up. */

void DoFinish (GPtr globals)
{	
	WriteScriptParams (globals); // writes our parameters to the scripting system
}

/*****************************************************************************/

/* This routine will always be called if DoStart does not return an error
   (even if DoContinue returns an error or the user aborts the operation).
   This allows the module to perform any needed cleanup.  None is required
   in this example. */

void AddComment (GPtr globals)
{
	
	/* We will attempt to add a resource showing that we made this change. */
	
	Str255 s, s0, s1, s2;
	Ptr p = NULL;
	int32 size = 0;
	Handle h;
	
	if (!ResourceProcsAvailable (NULL))
		return;
		
	p = PIGetResource(StringResource,
					  kHistoryEntry,
					  &size);
	/* Pulls a pascal string from the resource tree. */
	
	if (p != NULL && size > 0)
	{
		/* create "originHxoriginV" entry */
		DoubleToString(gOriginH, s0, kPrecision);
		DoubleToString(gOriginV, s, kPrecision);
		s0[ ++s0[0] ] = 'x';
		AppendString(s0, s, 1, s[0]);
		
		/* create "nudgeHxnudgeV" entry */
		DoubleToString(gNudgeH, s1, kPrecision);
		DoubleToString(gNudgeV, s, kPrecision);
		s1[ ++s1[0] ] = 'x';
		AppendString(s1, s, 1, s[0]);
		
		/* create "majorxminor" entry */
		DoubleToString(gGridMajor, s2, kPrecision);
		NumToString(gGridMinor, s);
		s2[ ++s2[0] ] = 'x';
		AppendString(s2, s, 1, s[0]);
		
		s[ (s[0]=0)+1 ] = 0;
		AppendString(s, (unsigned char *)p, 1, (short)size);
		s[ s[0] ] = 0;
		
		PIParamText(s, s0, s1, s2);
	
		h = PIString2Handle(s);
		
		if (h != NULL)
		{
			PILockHandle(h, FALSE);
			(void) AddPIResource (histResource, h);
			PIUnlockHandle(h);
			PIDisposeHandle(h);
			h = NULL;
		}
	}
	if (p != NULL) PIReleaseResource(p);
	p = NULL;
}

/*****************************************************************************/

short getProxyRect (Rect *l, short item)
{
	switch(item)
	{
		case iTop:
			return l->top;
			break;
		case iLeft:
			return l->left;
			break;
		case iBottom:
			return l->bottom;
			break;
		case iRight:
			return l->right;
			break;
	}
}

/*****************************************************************************/

void setProxyRect (Rect *l, short item, short value)
{
	switch(item)
	{
		case iTop:
			l->top = value;
			break;
		case iLeft:
			l->left = value;
			break;
		case iBottom:
			l->bottom = value;
			break;
		case iRight:
			l->right = value;
			break;
	}
}

/*****************************************************************************/

void scaleRect(Rect *l, short n, short d) {
	l->top = (l->top * n) / d;
	l->left = (l->left * n) / d;
	l->bottom = (l->bottom * n) / d;
	l->right = (l->right * n) / d;
}

void shrinkRect(Rect *l, short x, short y) {
	l->top += y;
	l->left += x;
	l->bottom -= y;
	l->right -= x;
}

void copyRect(Rect *l, Rect *r) {
	l->top = r->top;
	l->left = r->left;
	l->bottom = r->bottom;
	l->right = r->right;
}

/*****************************************************************************/

void CalcGlobalProxies (GPtr globals)
{
	Rect	imageRect;
	
	int16	selCenterH, selCenterV;
	int16	selLength, selWidth;
	int16	length, width;
	int16	length2, width2;

	length = (gOrigProxyRect.bottom - gOrigProxyRect.top); // display length
	length2 = length / 2;
	width = (gOrigProxyRect.right - gOrigProxyRect.left); // display width
	width2 = width / 2;
	
	selLength = gStuff->filterRect.bottom - gStuff->filterRect.top; // selected length
	selWidth = gStuff->filterRect.right - gStuff->filterRect.left; // selected width
	
	// get the center of the selected space
	selCenterV = gStuff->filterRect.top + (selLength / 2);
	selCenterH = gStuff->filterRect.left + (selWidth / 2);
	
	imageRect.top = selCenterV - (width2); // from center, set display rect
	imageRect.left = selCenterH - (length2);
	imageRect.bottom = imageRect.top+width;
	imageRect.right = imageRect.left+length;
	
	copyRect(&gProxyScaleRect, &imageRect);
	copyRect(&gStuff->inRect, &imageRect);
	copyRect(&gStuff->outRect, &imageRect);
	
}

/*****************************************************************************/

void SetupFilterRecordForProxy (GPtr globals, Rect proxyScaleRect, short scaleFactor)

	{
	
	// Rect	wholeImage;
	
	//CalcProxyScaleFactor (proxyScaleRect, &gProxyRect, &scaleFactor);	

	/*
	wholeImage.top = 0;
	wholeImage.left = 0;
	wholeImage.bottom = gStuff->imageSize.v;
	wholeImage.right = gStuff->imageSize.h;
	*/
	
	//wholeImage = gStuff->filterRect; // *************
	
	//copyRect (&gStuff->inRect, &wholeImage); // ***************
	//copyRect(&gStuff->inRect, &proxyScaleRect);
	/* ask for entire image size */
	
	//scaleRect (&gStuff->inRect, 1, scaleFactor);

	//copyRect (&gStuff->outRect, &proxyScaleRect);
	//copyRect(&gStuff->outRect, &wholeImage); // *************
	//scaleRect (&gStuff->outRect, 1, scaleFactor);
	/* but only show for filter rect */
		
	gStuff->inLoPlane = gStuff->outLoPlane = 0;
	gStuff->inHiPlane = gStuff->outHiPlane = gStuff->planes - 1;

	copyRect(&gStuff->maskRect, &gStuff->outRect);

	gStuff->inputRate = long2fixed(1L);
	gStuff->maskRate = gStuff->inputRate; //long2fixed(scaleFactor);
 
	gStuff->inputPadding = plugInWantsEdgeReplication;
	gStuff->outputPadding = gStuff->inputPadding;
	gStuff->maskPadding = gStuff->inputPadding;
	
	gResult = AdvanceState ();

	// Now gStuff->inData should have the subsampled version of input data
	// and gStuff->outData should be a buffer the size of input data that
	// could be used as a temporary buffer to do intermediate filtering
	// to display proxies

	}

/***********************************************************************************/

/* Computes the scaled down rectangle and the scale factor for the proxy */

void CalcProxyScaleFactor (Rect proxyScaleRect, Rect *proxyRect, short *scaleFactor)
	{

	short filterHeight;
	short filterWidth;
	short itemHeight;
	short itemWidth;
	Point fraction;

	shrinkRect (proxyRect, kGutter, kGutter); 		 // for the border

	filterHeight = proxyScaleRect.bottom - proxyScaleRect.top;
	filterWidth  = proxyScaleRect.right  - proxyScaleRect.left;
	
	itemHeight = proxyRect->bottom - proxyRect->top;
	itemWidth  = proxyRect->right  - proxyRect->left;
	
	if (itemHeight > filterHeight)
		itemHeight = filterHeight;
		
	if (itemWidth > filterWidth)
		itemWidth = filterWidth;
	
	fraction.h = ((filterWidth + itemWidth) / itemWidth)-1;
	fraction.v = ((filterHeight + itemHeight) / itemHeight)-1;

	if (fraction.h > fraction.v) 
		*scaleFactor = fraction.h;
	else
		*scaleFactor = fraction.v;

	copyRect (proxyRect, &proxyScaleRect);	
	scaleRect (proxyRect, 1, *scaleFactor);
	}

/*****************************************************************************/

/* Routines to handle the proxy */

void DoZoomRect (GPtr globals)
{
	if (gRepFactor < 1 && gRepFactor > 0)
		ZoomOutRect (globals);
	else if (gRepFactor >= 1)
		ZoomInRect (globals);
}
/*****************************************************************************/

/* Do our Zoom in.  Assumes gRepFactor >= 1 */

void ZoomInRect (GPtr globals)
{
	short totalColumns, totalRows, totalPlanes;
	short centerCol, centerRow;
	short offsetCol, offsetRow;
	
	short startCol, startRow;
	short endCol, endRow;
	
	short proxyCenterCol, proxyCenterRow;
	short proxyOffsetCol, proxyOffsetRow;
		
	short loopColumns = 1, loopRows = 0;
	short plane = 0, loopReps = 0, loopRepsRows = 0;
	long convert = 0;

	unsigned8 *srcPtr = (unsigned8 *) gStuff->inData;
	unsigned8 *dstPtr = (unsigned8 *) gStuff->outData;
	unsigned8 *maskPtr = (unsigned8 *) gStuff->maskData;
	
	//SetupFilterRecordForProxy (globals, gProxyScaleRect, loopColumns);
	
	totalColumns = gStuff->outRect.right - gStuff->outRect.left;
	totalRows = gStuff->outRect.bottom - gStuff->outRect.top;
	
	
	totalPlanes = gStuff->planes;
	centerCol = totalColumns / 2;
	centerRow = totalRows / 2;
	offsetCol = (short)(totalColumns / gRepFactor / 2);
	offsetRow = (short)(totalRows / gRepFactor / 2);

	startCol =  /* gProxyScaleRect.left = gStuff->outRect.left + */ centerCol - offsetCol;
	startRow = /* gProxyScaleRect.top = gStuff->outRect.top + */ centerRow - offsetRow;
	endCol =  /* gProxyScaleRect.right = gStuff->outRect.right - */ centerCol + offsetCol;
	endRow =  /* gProxyScaleRect.bottom = gStuff->outRect.bottom - */ centerRow + offsetRow;
	
	proxyCenterCol = gOrigProxyRect.left + 
						   ((gOrigProxyRect.right - gOrigProxyRect.left) / 2);
	proxyCenterRow = gOrigProxyRect.top + 
						   ((gOrigProxyRect.bottom - gOrigProxyRect.top) / 2);
	proxyOffsetCol = centerCol;
	proxyOffsetRow = centerRow;
	
	for (loopRows = startRow; loopRows < endRow; loopRows++)
	{
		for (loopRepsRows = 0; loopRepsRows < gRepFactor; loopRepsRows++)
		{
			srcPtr = (unsigned8 *) gStuff->inData + ((loopRows * gStuff->inRowBytes) + startCol * totalPlanes);
			convert = (((loopRows - startRow) * ((long)gRepFactor)) + loopRepsRows) * gStuff->outRowBytes;
			dstPtr = (unsigned8 *) gStuff->outData + convert;
			for (loopColumns = startCol; loopColumns < endCol; loopColumns++)
			{
				for (loopReps = 0; loopReps < gRepFactor; loopReps++)
				{
					for (plane = 0; plane < totalPlanes; plane++)
					{
						dstPtr [plane] = srcPtr [plane];
						if (gStuff->haveMask)
							; // transfer mask data here
					}
					dstPtr += totalPlanes;		
				}
				srcPtr += gStuff->inHiPlane - gStuff->inLoPlane + 1;
			}
		}
	}
	/* resize proxy rect */
	gProxyRect.left   = proxyCenterCol - proxyOffsetCol > gOrigProxyRect.left
					  ? proxyCenterCol - proxyOffsetCol : gOrigProxyRect.left;
	gProxyRect.right  = proxyCenterCol + proxyOffsetCol < gOrigProxyRect.right
					  ? proxyCenterCol + proxyOffsetCol : gOrigProxyRect.right;
	gProxyRect.top    = proxyCenterRow - proxyOffsetRow > gOrigProxyRect.top
					  ? proxyCenterRow - proxyOffsetRow : gOrigProxyRect.top;
	gProxyRect.bottom = proxyCenterRow + proxyOffsetRow < gOrigProxyRect.bottom
					  ? proxyCenterRow + proxyOffsetRow : gOrigProxyRect.bottom;
	/* up to calling party to use this data, if at all */	

}
	
/*****************************************************************************/

/* Do our Zoom Out.  Assumes gRepFactor between 0...1. */

void ZoomOutRect (GPtr globals)
{
	const unsigned8 kUIBackcolor = 230;
	
	short totalInColumns, totalInRows, totalInPlanes;
	short totalOutColumns, totalOutRows, totalOutPlanes;
	short centerCol, centerRow;
	short offsetCol, offsetRow;
	
	short startCol, startRow;
	short endCol, endRow;

	short skipVal;
	
	short proxyCenterCol, proxyCenterRow;
	short proxyOffsetCol, proxyOffsetRow;
	
	short loopColumns = 1, loopRows = 0;
	short plane = 0, loopReps = 0, loopRepsRows = 0;
	long convert = 0;

	unsigned8 *srcPtr = (unsigned8 *) gStuff->inData;
	unsigned8 *dstPtr = (unsigned8 *) gStuff->outData;
	unsigned8 *maskPtr = (unsigned8 *) gStuff->maskData;
	
	skipVal =  (short)(1/gRepFactor);

	//SetupFilterRecordForProxy (globals, gProxyScaleRect, loopColumns); //skipVal);

	totalOutColumns = gStuff->outRect.right - gStuff->outRect.left;
	totalInColumns = gStuff->inRect.right - gStuff->inRect.left;
	totalOutRows =  gStuff->outRect.bottom - gStuff->outRect.top;
	totalInRows = gStuff->inRect.bottom - gStuff->inRect.top;
	totalInPlanes = gStuff->inHiPlane - gStuff->inLoPlane + 1;
	totalOutPlanes = gStuff->planes;
	centerCol = totalOutColumns / 2;
	centerRow = totalOutRows / 2;
	offsetCol = (short)(totalOutColumns / gRepFactor / 2);
	offsetRow = (short)(totalOutRows / gRepFactor / 2);

	startCol =  /* gProxyScaleRect.left = gStuff->outRect.left + */ centerCol - offsetCol;
	startRow = /* gProxyScaleRect.top = gStuff->outRect.top + */ centerRow - offsetRow;
	endCol =  /* gProxyScaleRect.right = gStuff->outRect.right - */ centerCol + offsetCol;
	endRow =  /* gProxyScaleRect.bottom = gStuff->outRect.bottom - */ centerRow + offsetRow;

	proxyCenterCol = gOrigProxyRect.left + 
						   ((gOrigProxyRect.right - gOrigProxyRect.left) / 2);
	proxyCenterRow = gOrigProxyRect.top + 
						   ((gOrigProxyRect.bottom - gOrigProxyRect.top) / 2);
	proxyOffsetCol = (short)((totalOutColumns * gRepFactor) / 2);
	proxyOffsetRow = (short)((totalOutRows * gRepFactor) / 2);
	
	// FIXME:  This doesn't work right.  Fix later.
	for (loopRows = 0; loopRows < totalInRows; loopRows += skipVal)
	{
		srcPtr = (unsigned8 *) gStuff->inData + (loopRows * gStuff->inRowBytes);
		convert = (long)(loopRows * gRepFactor) * gStuff->outRowBytes;
		dstPtr = (unsigned8 *) gStuff->outData + convert;
		
		for (loopColumns = 0; loopColumns < totalInColumns; loopColumns += skipVal)
		{
			for (plane = 0; plane < totalOutPlanes; plane++)
			{
				dstPtr [plane] = srcPtr [plane];
				if (gStuff->haveMask)
					; // transfer mask data here
			}
			dstPtr += totalOutPlanes;
			srcPtr += gStuff->inHiPlane - gStuff->inLoPlane + 1; //((gStuff->inHiPlane - gStuff->inLoPlane)*skipVal)+1;
		}

		for (loopColumns = (short)(totalInColumns * gRepFactor); loopColumns < totalOutColumns; loopColumns++)
		{ // clear out remaining pixels to end of line
			for (plane = 0; plane < totalOutPlanes; plane++)
			{
				dstPtr [plane] = kUIBackcolor; // gStuff->backColor [plane]; // reset to whatever
				// should reset this color to whatever the real UI background color is
				if (gStuff->haveMask)
					; // clear mask here
			}
			dstPtr += totalOutPlanes;
		}
	}
	for (loopRows = (short)(totalInRows * gRepFactor); loopRows < totalOutRows; loopRows++)
	{ // clear out remaining rows to end of pix
		dstPtr = (unsigned8 *) gStuff->outData + (loopRows * gStuff->outRowBytes);
		for (loopColumns = 0; loopColumns < totalOutColumns; loopColumns++)
		{
			for (plane = 0; plane < totalOutPlanes; plane++)
			{
				dstPtr [plane] = kUIBackcolor; // gStuff->backColor [plane]; // reset to whatever
				// should reset this color to whatever the real UI background color is
				if (gStuff->haveMask)
					; // clear mask here
			}
			dstPtr += totalOutPlanes;
		}
	} 

	/* resize proxy rect */
	gProxyRect.left   = proxyCenterCol - proxyOffsetCol > gOrigProxyRect.left
					  ? proxyCenterCol - proxyOffsetCol : gOrigProxyRect.left;
	gProxyRect.right  = proxyCenterCol + proxyOffsetCol < gOrigProxyRect.right
					  ? proxyCenterCol + proxyOffsetCol : gOrigProxyRect.right;
	gProxyRect.top    = proxyCenterRow - proxyOffsetRow > gOrigProxyRect.top
					  ? proxyCenterRow - proxyOffsetRow : gOrigProxyRect.top;
	gProxyRect.bottom = proxyCenterRow + proxyOffsetRow < gOrigProxyRect.bottom
					  ? proxyCenterRow + proxyOffsetRow : gOrigProxyRect.bottom;
	/* up to calling party to use this data, if at all */	
	
}
	
/*****************************************************************************/
/* Zooms in by one factor (parts of 1.0 until 1.0, then 1.0, 2.0, etc. */

void ZoomIn (short *zoom)
{
	if (*zoom >= kNumZooms)
		*zoom += kNumZooms;
	else (*zoom)++;
}

/* Same thing but zooms out. */

void ZoomOut (short *zoom)
{
	if (*zoom >= (kNumZooms*2))
		*zoom -= kNumZooms;
	else (*zoom)--;
}

/*****************************************************************************/
/* Takes an enumeration and returns corresponding zoom size */

short GetZoomFactor (GPtr globals)

{
	
	short	sub = 0;
	short	returnValue = noErr;
	
	/* check for zoomFactor bounds */
	
	if (gZoomFactor <= kMinZoom)
	{
		gZoomFactor = kMinZoom;
		returnValue = hitLowBounds;
	}
	else if (gZoomFactor >= kMaxZoom)
	{
		gZoomFactor = kMaxZoom;
		returnValue = hitHighBounds;
	}
	
	sub = gZoomFactor % kNumZooms;
	
	switch (sub)
		{
			case 1:
				gRepFactor = (float)0.25;
				break;
			case 2:
				gRepFactor = (float)0.33;
				break;
			case 3:
				gRepFactor = (float)0.50;
				break;
			case 4:
				gRepFactor = (float)0.66;
				break;
			case 5:
				gRepFactor = (float)0.75;
				break;
			default:
				gRepFactor = (float)0;
				break;
		}
	
		gRepFactor += gZoomFactor / kNumZooms;
	
	return returnValue;
}

/*****************************************************************************/

static void TrimVRect (VRect *dst, VRect *src)
	{
	
	if (dst->top    < src->top   ) dst->top    = src->top;
	if (dst->left   < src->left  ) dst->left   = src->left;
	if (dst->bottom > src->bottom) dst->bottom = src->bottom;
	if (dst->right  > src->right ) dst->right  = src->right;
	
	}

/*****************************************************************************/

static Boolean EqualVRects (VRect *a, VRect *b)
	{
	
	return  a->top    == b->top
		 && a->left   == b->left
		 && a->bottom == b->bottom
		 && a->right  == b->right;
	
	}

/*****************************************************************************/

static int32 AccountChannel (ReadChannelDesc *source,
									  ReadChannelDesc *mask,
									  WriteChannelDesc *dest)
	{
	
	VRect limit;
	
	limit = source->bounds;
	TrimVRect (&limit, &dest->bounds);
	if (mask != NULL) TrimVRect (&limit, &mask->bounds);
		
	if (limit.right <= limit.left || limit.bottom <= limit.top) return 0;
	
	return (limit.right - limit.left) * (limit.bottom - limit.top);
	
	}

/*****************************************************************************/

static void ApplyChannel (GPtr globals,
						  ReadChannelDesc *source,
						  PixelMemoryDesc *sDesc,
						  ReadChannelDesc *mask, 
						  PixelMemoryDesc *mDesc,
						  WriteChannelDesc *dest, 
						  ChannelReadPort destRead,
						  PixelMemoryDesc *dDesc,
						  PixelMemoryDesc *rDesc,
						  int32 *done,
						  int32 total)
	{
	
	VRect limit;
	int32 row, col, row2, col2;
	unsigned8 *s, *m, *d;
	unsigned8 *r = (unsigned8 *)rDesc->data;
	Boolean initRandom = (*r == kInitRandom);
		
	limit = source->bounds;
	TrimVRect (&limit, &dest->bounds);
	if (mask != NULL) TrimVRect (&limit, &mask->bounds);
		
	if (limit.right <= limit.left || limit.bottom <= limit.top) return;
	
	for (row = limit.top; row < limit.bottom; row += kBlockRows)
		for (col = limit.left; col < limit.right; col += kBlockCols)
			{
			
			VRect area;
			PSScaling scaling;
			VRect wrote;
			
			if (TestAbort ())
				{
				gResult = userCanceledErr;
				return;
				}
			
			area.top = row;  area.bottom = row + kBlockRows;
			area.left = col; area.right  = col + kBlockCols;
			
			if (limit.bottom < area.bottom) area.bottom = limit.bottom;
			if (limit.right  < area.right ) area.right  = limit.right;
			
			scaling.sourceRect = area;
			scaling.destinationRect = area;
			
			gResult = ReadPixels (destRead, &scaling, &area, dDesc, &wrote);
			if (gResult != noErr) return;
			
			if (!EqualVRects (&area, &wrote))
				{
				gResult = -1;
				return;
				}
				
			gResult = ReadPixels (source->port, &scaling, &area, sDesc, &wrote);
			if (gResult != noErr) return;
			
			if (!EqualVRects (&area, &wrote))
				{
				gResult = -1;
				return;
				}
				
			if (mask != NULL)
				{
				
				gResult = ReadPixels (mask->port, &scaling, &area, mDesc, &wrote);
				if (gResult != noErr) return;
				
				if (!EqualVRects (&area, &wrote))
					{
					gResult = -1;
					return;
					}
				}
				m = (unsigned8 *) mDesc->data;
				/* mask all set and ready to go */
									
			/* heart of the routine.  Compares source pixel and destination
			   pixel and, if destination is smaller, replaces it with
			   source */

			s = (unsigned8 *) sDesc->data;
			d = (unsigned8 *) dDesc->data;
			r = (unsigned8 *) rDesc->data;
						
			for (row2 = kBlockRows; row2 > 0; --row2)
				for (col2 = kBlockCols; col2 > 0; --col2)
				{
					
					if (mask != NULL && *m < *s ) *s = *m;
							if (*s > 127) *d = *s;
							else *d = 0;
					
					if (mask != NULL && *m < *s ) *s = *m;
							if (*s < 128) *d = 255 - *s; // if (*d < *s) *d = *s;
							else *d = 0;
					
					if (initRandom)
							{
								if ((((unsigned16) Random ()) % 100) < gPercent)
									*r = kRandomOn; // flag as picked
								else *r = kRandomOff; // flag as off
							}
							if (*r == kRandomOn)
								*d = 255;
							else *d = 0;
					++s;
					++d;
					++r;
					if (mask != NULL) ++m;
				}
					
			gResult = WritePixels (dest->port, &area, dDesc);
			if (gResult != noErr) return;
			
			*done += (area.right - area.left) * (area.bottom - area.top);
			
			UpdateProgress (*done, total);
				
			}
	
	}

/*****************************************************************************/

void BuildProxy (GPtr globals)
{
	
	int32 total = 0;
	int32 done = 0;
	Ptr		p = 0;
	
	const Boolean DoTarget = false;
	
	BufferID sBuffer = 0;
	Ptr sData = NULL;
	BufferID dBuffer = 0;
	Ptr dData = NULL;
	BufferID mBuffer = 0;
	Ptr mData = NULL;
	BufferID rBuffer = 0;
	Ptr rData = NULL;
	
	PixelMemoryDesc sDesc, dDesc, mDesc, rDesc;
	
	ReadImageDocumentDesc *doc = gStuff->documentInfo;
	ChannelReadPort selectionRead;
	WriteChannelDesc actualSelection;
	WriteChannelDesc *selection = &actualSelection;
	//WriteChannelDesc *selection = gStuff->newSelection;
	ReadChannelDesc *composite;
	ReadChannelDesc *transparency;
	ReadChannelDesc *curChannel;
	
	
	/* Access the port procs. */
	
	if (!WarnChannelPortAvailable ())
		{
		gResult = errPlugInHostInsufficient;
		goto CleanUp;
		}
			
	/* We will need a read port for the selection. */
	
	gResult = ReadFromWritePort(&selectionRead, selection->port);
	if (gResult != noErr) goto CleanUp;
	
	/* Figure out which composite data to use. */
	
	if (DoTarget)
	{
		composite = doc->targetCompositeChannels;
		transparency = doc->targetTransparency;
	
		if (composite == NULL)
		{
			composite = doc->mergedCompositeChannels;
			transparency = doc->mergedTransparency;
		}
	}
	else
	{
		composite = doc->mergedCompositeChannels;
		transparency = doc->mergedTransparency;
	}
			
	/* Allocate the buffers. */
	
	if (!WarnBufferProcsAvailable ())
		{
		gResult = +1;
		goto CleanUp;
		}
		
	#define kBufferSize kBlockRows * (long) kBlockCols
	
	gResult = AllocateBuffer (kBufferSize, &sBuffer);
	if (gResult != noErr) goto CleanUp;
	
	gResult = AllocateBuffer (kBufferSize, &dBuffer);
	if (gResult != noErr) goto CleanUp;
	
	gResult = AllocateBuffer (kBufferSize, &rBuffer);
	if (gResult != noErr) goto CleanUp;
	
	if (transparency != NULL)
		{
		gResult = AllocateBuffer (kBufferSize, &mBuffer);
		if (gResult != noErr) goto CleanUp;
		}
		
	/* Lock the buffers down. */
	
	sData = LockBuffer (sBuffer, false);
	dData = LockBuffer (dBuffer, false);
	rData = LockBuffer (rBuffer, false);
	if (mBuffer != 0) mData = LockBuffer (mBuffer, false);
	
	/* Set up the pixel memory descriptors. */
	
	sDesc.data = sData;
	sDesc.rowBits = kBlockCols * 8;
	sDesc.colBits = 8;
	sDesc.bitOffset = 0;
	sDesc.depth = 8;
	
	dDesc = sDesc;
	dDesc.data = dData;
	
	rDesc = sDesc;
	rDesc.data = rData;
	
	mDesc = sDesc;
	mDesc.data = mData;
	
	rData[0] = kInitRandom; // flag for uninitialized random buffer
	
	/* Count the channels to process. */
	
	if (doc->selection != NULL)
		total += AccountChannel (doc->selection, NULL, selection);
	
	curChannel = composite;
	while (curChannel != NULL)
	{
		if (DoTarget ? curChannel->target : curChannel->shown)
			total += AccountChannel (curChannel, transparency, selection);
									  
		curChannel = curChannel->next;
	}
		

	if (doc->targetLayerMask != NULL && 
	   (DoTarget ? doc->targetLayerMask->target : doc->targetLayerMask->shown))
			total += AccountChannel (doc->targetLayerMask, NULL, selection);
			
	curChannel = doc->alphaChannels;
	while (curChannel != NULL)
	{
		if (DoTarget ? curChannel->target : curChannel->shown)
			total += AccountChannel (curChannel, NULL, selection);
							  
		curChannel = curChannel->next;
	}

	
	/* Apply any existing selection. */
	
	if (doc->selection != NULL)
		{
		ApplyChannel (globals, doc->selection, &sDesc,
						  NULL, &mDesc,
						  selection, selectionRead, &dDesc,
						  &rDesc, &done, total);
		if (gResult != noErr) goto CleanUp;
		}
	
	/* Apply each of the channels. This isn't the most efficient
	approach since it reads the data back out of the selection
	repeatedly, but it provides a good work out for the code. */
	
	curChannel = composite;
	while (curChannel != NULL)
	{
		if (DoTarget ? curChannel->target : curChannel->shown)
		{
			ApplyChannel (globals, curChannel, &sDesc,
							  transparency, &mDesc,
							  selection, selectionRead, &dDesc,
							  &rDesc, &done, total);
			if (gResult != noErr) goto CleanUp;
		}
							  
		curChannel = curChannel->next;
	}
		
	/* Apply the layer mask. */
	if (doc->targetLayerMask != NULL &&
	   (DoTarget ? doc->targetLayerMask->target : doc->targetLayerMask->shown))
	{
		ApplyChannel (globals, doc->targetLayerMask, &sDesc,
						  NULL, &mDesc,
						  selection, selectionRead, &dDesc,
						  &rDesc, &done, total);
		if (gResult != noErr) goto CleanUp;
	}
		
	/* Process the alpha channels. */
	curChannel = doc->alphaChannels;
	while (curChannel != NULL)
	{
		if (DoTarget ? curChannel->target : curChannel->shown)
		{
			ApplyChannel (globals, curChannel, &sDesc,
							  NULL, &mDesc,
							  selection, selectionRead, &dDesc,
							  &rDesc, &done, total);
			if (gResult != noErr) goto CleanUp;
		}
							  
		curChannel = curChannel->next;
	}
	
	CleanUp:
	
	if (sData != NULL) UnlockBuffer (sBuffer);
	if (dData != NULL) UnlockBuffer (dBuffer);
	if (mData != NULL) UnlockBuffer (mBuffer);
	if (rData != NULL) UnlockBuffer (rBuffer);
	
	if (sBuffer != 0) FreeBuffer (sBuffer);
	if (dBuffer != 0) FreeBuffer (dBuffer);
	if (mBuffer != 0) FreeBuffer (mBuffer);	
	if (rBuffer != 0) FreeBuffer (rBuffer);
	
	WriteScriptParams(globals);

}