/*
	File: Propetizer.h

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	Utilities for Propetizer example filter module.
*/

#ifndef __Propetizer_H__
#define __Propetizer_H__

#include "PIFilter.h"
#include "PIUtilities.h"
#include "PIProperties.h"

/*****************************************************************************/

#define kMinNumber			-32767
#define kMaxNumber			32767

#define kOriginMin			0
#define kOriginMax			999999

#define kNudgeMin			0
#define kNudgeMax			999999

#define kGridMin			0
#define kGridMax			999999

#define kCopyMin			0
#define kCopyMax			255

#define	kNumZooms			6
#define kMinZoom			1
#define kMaxZoom			kNumZooms*8

#define kPrecision			6
#define kGutter				2 // 2 pixel gutter for proxy

#define keyNudgeH			'nudH'
#define keyNudgeV			'nudV'
#define keyGridMajor		'grMJ'
#define keyGridMinor		'grMN'
#define keyCopyright		'cpyR'
#define keyURL				'urlL'

#define histResource		'hist'
#define kHistoryEntry		16989

#define kUnits				16100

#define updateButton		3
#define revertButton		4
#define proxyItem			5
#define iRepFactor			6
#define minusButton			7
#define plusButton			8

#define kInitRandom			1
#define kRandomOn			255
#define kRandomOff			0

#define kBlockRows			64
#define kBlockCols			64

/*****************************************************************************/

enum
{
	iPropTitle = 10,
	iPropSerialString,
	iPropRulerUnits,
	iPropBigNudgeH,
	iPropBigNudgeV,
	iPropRulerOriginH,
	iPropRulerOriginV,
	iPropGridMajor,
	iPropGridMinor,
	iPropCopyright,
	iPropURL,
};

#define kProp1 		iPropTitle
#define kPropStart	iPropBigNudgeH
#define kPropEnd	iPropURL
	
enum
{	
	iTop,
	iLeft,
	iBottom,
	iRight
};

enum
{
	iRulerUnit_pixels,
	iRulerUnit_inches,
	iRulerUnit_cm,
	iRulerUnit_points,
	iRulerUnit_picas
};

/*****************************************************************************/

/* Proxy error values */

enum
{
	hitLowBounds = 1,
	hitHighBounds
};

/*****************************************************************************/

typedef struct TParameters
	{
	short	percent;
	short	queryForParameters;
	double	nudgeH, nudgeV, originH, originV, gridMajor;
	long	gridMinor, copyright;
	Str255	url;
	OSType	disposition;
	Rect	proxyScaleRect;
	} TParameters, *PParameters, **HParameters;

#define gParams				((PParameters) *gStuff->parameters)
#define gPercent 			(gParams->percent)
#define gQueryForParameters (gParams->queryForParameters)
#define gDisposition 		(gParams->disposition)
#define gProxyScaleRect 	(gParams->proxyScaleRect)
#define	gNudgeH				(gParams->nudgeH)
#define gNudgeV				(gParams->nudgeV)
#define gOriginH 			(gParams->originH)
#define gOriginV 			(gParams->originV)
#define gGridMajor 			(gParams->gridMajor)
#define gGridMinor 			(gParams->gridMinor)
#define gCopyright			(gParams->copyright)
#define gURL				(gParams->url)

/* special case for proxy builder */
#define rdProxyScaleRect stuff->outRect
//#define rdProxyScaleRect ((PParameters) *stuff->parameters)->proxyScaleRect

/*****************************************************************************/

extern Handle hDllInstance;

typedef struct Globals
{
	short 	result;
	Rect	proxyRect, origProxyRect;
	float	repFactor;
	short	zoomFactor;
	FilterRecord *filterParamBlock;
} Globals, *GPtr, **GHdl;
	
#define gResult 		(globals->result)
#define gStuff  		(globals->filterParamBlock)
#define gProxyRect		(globals->proxyRect)
#define gOrigProxyRect	(globals->origProxyRect)
#define gRepFactor		(globals->repFactor)
#define gZoomFactor		(globals->zoomFactor)

/*****************************************************************************/

void DoAbout (GPtr globals);
void scaleRect(Rect *l, short n, short d);
void shrinkRect(Rect *l, short x, short y);
void copyRect(Rect *l, Rect *r);
short PromptUserForInput (GPtr globals);
void CalcProxyScaleFactor (Rect scaleRect, Rect *proxyRect, short *scaleFactor);
Boolean CheckProxiesSupported (DisplayPixelsProc displayPixels);
void DoFilterRect (GPtr globals);
void DoZoomRect (GPtr globals);
void SetupFilterRecordForProxy (GPtr globals, Rect proxyScaleRect, short scaleFactor);	 
void BuildProxy (GPtr globals);
short GetPropUnits (OSType key);
OSType GetPropKey (short propEnum);
void ValidateParameters (GPtr globals);
void setProxyRect (Rect *l, short item, short value);
short getProxyRect (Rect *l, short item);
short GetZoomFactor (GPtr globals);
Boolean ReadScriptParams (GPtr globals);
OSErr WriteScriptParams (GPtr globals);
void PropertiesToGlobals(GPtr globals);
void GlobalsToProperties(GPtr globals);
void AddComment (GPtr globals);
void BuildProxy (GPtr globals);
void ZoomOut (short *zoom);
void ZoomIn (short *zoom);
void CalcGlobalProxies (GPtr globals);

/*****************************************************************************/

#endif /* __Propetizer_H__ */
