/*
	File: Propetizer.r

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	Rez source file for Propetizer example.
*/

/**********************************************************************************/
/* Defines needed in other rez files */

#define plugInName			"Propetizer"
#define VersionString		"4.0"

/********************************************************************************/

#if Macintosh
#include "Types.r"
#include "SysTypes.r"
#include "PIGeneral.r"
#include "PIUtilities.r"
#include "DialogUtilities.r"
#endif

#if MSWindows
#include "PIGeneral.h"
#include "PIUtilities.r"
#include "WinDialogUtils.r"
#endif

#include "PIActions.h"

/********************************************************************************/

#define plugInComment 		"propetizer example filter plug-in"
#define vendorName			"AdobeSDK"

#define keyNudgeH			'nudH'
#define keyNudgeV			'nudV'
#define keyGridMajor		'grMJ'
#define keyGridMinor		'grMN'
#define keyCopyright		'cpyR'
#define keyURL				'urlL'
#define ourSuiteID			'sdK9'
#define ourClassID			ourSuiteID
#define ourEventID			'griD'

#define kHistoryEntry		16989
#define kUnits				16100

/********************************************************************************/

resource 'PiPL' (ResourceID, purgeable)
	{
		{
		Kind { Filter },
		Name { plugInName "..." },
		Category { "Utilities" },
		Version { (latestFilterVersion << 16) | latestFilterSubVersion },

		#if Macintosh
		Code68K { Filter, $$ID },
		CodePowerPC { 0, 0, "" },
		#endif

		#if MSWindows
		CodeWin32X86 { "ENTRYPOINT" },
		#endif
		
		HasTerminology { ourClassID, ourEventID, ResourceID, "" },
		/* classID, eventID, aete ID, uniqueString */
		
		SupportedModes
			{
			noBitmap, doesSupportGrayScale,
			noIndexedColor, doesSupportRGBColor,
			doesSupportCMYKColor, doesSupportHSLColor,
			doesSupportHSBColor, doesSupportMultichannel,
			doesSupportDuotone, doesSupportLABColor
			},

		EnableInfo { "in (PSHOP_ImageMode, GrayScaleMode, RGBMode,"
					     "CMYKMode, HSLMode, HSBMode, MultichannelMode,"
					     "DuotoneMode, LabMode)" },
					     
		FilterCaseInfo
			{
				{
				/* Flat data, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Flat data with selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
				
				/* Floating selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Editable transparency, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Editable transparency, with selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Preserved transparency, no selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination,
					
				/* Preserved transparency, with selection */
				inWhiteMat, outStraightData,
				doNotWriteOutsideSelection,
				filtersLayerMasks, worksWithBlankData,
				copySourceToDestination
				}
			}
		}
	};

resource 'PiMI' (ResourceID, purgeable)
{
	latestFilterVersion,
	latestFilterSubVersion,
	0,
	supportsGrayScale +
	supportsRGBColor +
	supportsCMYKColor +
	supportsHSLColor +
	supportsHSBColor +
	supportsMultichannel +
	supportsDuotone +
	supportsLABColor,
	'    ', /* No required host */
	{},
};

/* About box resources */

resource StringResource (AboutID, "About Text", purgeable)
{
	plugInName "\n\n"
	"Version " VersionString " "
	"Release " ReleaseString "\n"
	"Copyright  1996, Adobe Systems Incorporated.\n"
	"All rights reserved.\n\n"
	"An example plug-in filter showing properties in Adobe Photoshop."
};

/* History resource */

resource StringResource (kHistoryEntry, "History", purgeable)
{
	plugInName ": grid=^2, nudge=^1, origin=^0."
};

/* Parameters dialog box */

resource 'DLOG' (ResourceID+1, "UI", purgeable)
{
	{102, 148, 450, 500},
	movableDBoxProc,
	visible,
	noGoAway,
	0x0,
	ResourceID+1,
	plugInName
};

resource 'DITL' (ResourceID+1, "UI", purgeable)
{
	{	/* array DITLarray: 20 elements */
		/* [1] */
		{8, 274, 28, 334},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{40, 274, 60, 334},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{8, 196, 28, 256},
		Button {
			enabled,
			"Update"
		},
		/* [4] */
		{40, 196, 60, 256},
		Button {
			enabled,
			"Revert"
		},
		/* [5] */
		{8, 47, 64, 115},
		UserItem {
			enabled
		},
		/* [6] */
		{71, 63, 86, 102},
		StaticText {
			disabled,
			"100%"
		},
		/* [7] */
		{70, 44, 86, 60},
		Button {
			enabled,
			"-"
		},
		/* [8] */
		{70, 102, 86, 118},
		Button {
			enabled,
			"+"
		},
		/* [9] */
		{98, 4, 348, 160},
		StaticText {
			disabled,
			"propTitle:\n\npropSerialString:\n\npropRuler"
			"Units:\n\npropBigNudgeH/V:\n\npropRulerOrigi"
			"nH/V:\n\npropGridMajor/Minor:\n\npropCopyrig"
			"ht:\n\npropURL:"
		},
		/* [10] */
		{98, 164, 120, 342},
		StaticText {
			enabled,
			""
		},
		/* [11] */
		{130, 164, 152, 342},
		StaticText {
			enabled,
			""
		},
		/* [12] */
		{162, 164, 184, 249},
		StaticText {
			enabled,
			""
		},
		/* [13] */
		{192, 164, 214, 249},
		EditText {
			enabled,
			""
		},
		/* [14] */
		{192, 256, 214, 341},
		EditText {
			enabled,
			""
		},
		/* [15] */
		{224, 164, 246, 249},
		EditText {
			enabled,
			""
		},
		/* [16] */
		{224, 256, 246, 341},
		EditText {
			enabled,
			""
		},
		/* [17] */
		{256, 164, 278, 249},
		EditText {
			enabled,
			""
		},
		/* [18] */
		{256, 256, 278, 341},
		EditText {
			enabled,
			""
		},
		/* [19] */
		{288, 164, 310, 249},
		EditText {
			enabled,
			""
		},
		/* [20] */
		{320, 164, 342, 342},
		EditText {
			enabled,
			""
		}
	}
};

resource StringResource(kUnits, purgeable)
{
	"pixels"
};

resource StringResource (kUnits+1, purgeable)
{
	"inches"
};

resource StringResource (kUnits+2, purgeable)
{
	"cm"
};

resource StringResource (kUnits+3, purgeable)
{
	"points"
};

resource StringResource (kUnits+4, purgeable)
{
	"picas"
};
	
/* Scripting resource */

resource 'aete' (ResourceID, purgeable)
{
	1, 0, english, roman,									/* aete version and language specifiers */
	{
		"AdobeSDK",											/* vendor suite name */
		"Adobe example plug-ins",							/* optional description */
		ourSuiteID,											/* suite ID */
		1,													/* suite code, must be 1 */
		1,													/* suite level, must be 1 */
		{													/* structure for filters */
			"AdobeSDK gridproxy",							/* unique filter name */
			"gridproxy properties plug-in filter",			/* optional description */
			ourClassID,										/* class ID, must be unique or Suite ID */
			ourEventID,										/* event ID, must be unique */
			
			NO_REPLY,										/* never a reply */
			IMAGE_DIRECT_PARAMETER,							/* direct parameter, used by Photoshop */
			{												/* parameters here, if any */
				"nudge H",									/* parameter name */
				keyNudgeH,									/* parameter key ID */
				typeFloat,									/* parameter type ID */
				"horizontal big nudge",						/* optional description */
				flagsSingleParameter,						/* parameter flags */
				
				"nudge V",									/* parameter name */
				keyNudgeV,									/* parameter key ID */
				typeFloat,									/* parameter type ID */
				"vertical big nudge",						/* optional description */
				flagsSingleParameter,						/* parameter flags */

				"origin H",									/* parameter name */
				keyHorizontal,								/* parameter key ID */
				typeFloat,									/* parameter type ID */
				"horizontal origin",						/* optional description */
				flagsSingleParameter,						/* parameter flags */
				
				"origin V",									/* parameter name */
				keyVertical,								/* parameter key ID */
				typeFloat,									/* parameter type ID */
				"vertical origin",							/* optional description */
				flagsSingleParameter,						/* parameter flags */
				
				"gridMajor",								/* parameter name */
				keyGridMajor,								/* parameter key ID */
				typeFloat,									/* parameter type ID */
				"major grid divisions",						/* optional description */
				flagsSingleParameter,						/* parameter flags */
				
				"gridMinor",								/* parameter name */
				keyGridMinor,								/* parameter key ID */
				typeInteger,								/* parameter type ID */
				"minor grid subdivisions",					/* optional description */
				flagsSingleParameter,						/* parameter flags */
				
				"copyright",								/* you get the idea */
				keyCopyright,
				typeText,
				"copyright string",
				flagsSingleParameter,
				
				"url",
				keyURL,
				typeText,
				"URL string",
				flagsSingleParameter
				
			}
		},
		{},													/* non-filter plug-in class here */
		{},													/* comparison ops (not supported) */
		{}													/* any enumerations */
	}
};