/*
	File: PropetizerScripting.c

	Copyright (c) 1996, Adobe Systems Incorporated.
	All rights reserved.

	C source file for scripting functions for Propetizer example.
*/

#include "Propetizer.h"

/*****************************************************************************/
/* Checks the parameters against scripting-returned parameters, if any, and
   updates our parameters to match ones given to us by the scripting system. */

Boolean ReadScriptParams (GPtr globals)
{
	PIReadDescriptor			token = NULL;
	DescriptorKeyID				key = 0;
	DescriptorTypeID			type = 0;
	DescriptorKeyIDArray		array = { keyNudgeH, keyNudgeV,
										  keyHorizontal, keyVertical,
										  keyGridMajor, keyGridMinor,
										  keyCopyright, keyURL, NULLID };
	int32						flags = 0;
	Handle						h = NULL;
	OSErr						gotErr = noErr, stickyError = noErr;
	Boolean						returnValue = true;
	
	if (DescriptorAvailable())
	{
		token = OpenReader(array);
		if (token)
		{
			while (PIGetKey(token, &key, &type, &flags))
			{
				switch (key)
				{
					case keyNudgeH:
						PIGetFloat(token, &gNudgeH);
						break;
					case keyNudgeV:
						PIGetFloat(token, &gNudgeV);
						break;
					case keyHorizontal:
						PIGetFloat(token, &gOriginH);
						break;
					case keyVertical:
						PIGetFloat(token, &gOriginV);
						break;
					case keyGridMajor:
						PIGetFloat(token, &gGridMajor);
						break;
					case keyGridMinor:
						PIGetInt(token, &gGridMinor);
						break;
					case keyCopyright:
						PIGetInt(token, &gCopyright);
						break;
					case keyURL:
						PIGetText(token, &h);
						PIHandle2String(h, gURL);
						if (h != NULL) PIDisposeHandle(h);
						h = NULL;
						break;
					// ignore all other cases and classes
				}
			}

			stickyError = CloseReader(&token); // we're done, dispose.
			
			if (stickyError)
			{
				if (stickyError == errMissingParameter)
					;
					/* (descriptorKeyIDArray != NULL) Missing a parameter.
				       Walk IDarray for which one. */
				else
					gResult = stickyError; // real error occurred
			}
		}		
		gQueryForParameters = returnValue = PlayDialog();
		/* return TRUE if want to show our Dialog */			
	}
	return returnValue;
}

/*****************************************************************************/
/* Writes our parameters to the scripting system. */

OSErr WriteScriptParams (GPtr globals)
{
	PIWriteDescriptor			token = nil;
	OSErr						gotErr = noErr;
	Handle						hURL = PIString2Handle(gURL);
	
	if (DescriptorAvailable())
	{
		token = OpenWriter();
		if (token)
		{					
			PIPutFloat(token, keyHorizontal, &gNudgeH);
			PIPutFloat(token, keyVertical, &gNudgeV);
			PIPutFloat(token, keyHorizontal, &gOriginH);
			PIPutFloat(token, keyVertical, &gOriginV);
			PIPutFloat(token, keyGridMajor, &gGridMajor);
			PIPutInt(token, keyGridMinor, gGridMinor);
			PIPutInt(token, keyCopyright, gCopyright);
			PIPutText(token, keyURL, hURL);
			gotErr = CloseWriter(&token); // closes and sets to dialogOptional
			/* done.  Now pass handle on to Photoshop */
			if (hURL) PIDisposeHandle(hURL);

		}
	}
	return gotErr;
}

/*****************************************************************************/
/* Takes the property units and returns a corresponding enumerated number */

short GetPropUnits (OSType key)

{
	switch (key)
	{
		case propTitle:
			return iPropTitle;
			break;
		case propSerialString:
			return iPropSerialString;
			break;
		case propRulerUnits:
			return iPropRulerUnits;
			break;
		case propBigNudgeH:
			return iPropBigNudgeH;
			break;
		case propBigNudgeV:
			return iPropBigNudgeV;
			break;
		case propRulerOriginH:
			return iPropRulerOriginH;
			break;
		case propRulerOriginV:
			return iPropRulerOriginV;
			break;
		case propGridMajor:
			return iPropGridMajor;
			break;
		case propGridMinor:
			return iPropGridMinor;
			break;
		case propCopyright:
			return iPropCopyright;
			break;
		case propURL:
			return iPropURL;
			break;
	}
}

/*****************************************************************************/
/* Takes the property enumeration and returns the corresponding key */

OSType GetPropKey (short keyEnum)

{

	switch (keyEnum)
	{
		case iPropTitle:
			return propTitle;
			break;
		case iPropSerialString:
			return propSerialString;
			break;
		case iPropRulerUnits:
			return propRulerUnits;
			break;
		case iPropBigNudgeH:
			return propBigNudgeH;
			break;
		case iPropBigNudgeV:
			return propBigNudgeV;
			break;
		case iPropRulerOriginH:
			return propRulerOriginH;
			break;
		case iPropRulerOriginV:
			return propRulerOriginV;
			break;
		case iPropGridMajor:
			return propGridMajor;
			break;
		case iPropGridMinor:
			return propGridMinor;
			break;
		case iPropCopyright:
			return propCopyright;
			break;
		case iPropURL:
			return propURL;
			break;
	}
}

/*****************************************************************************/
/* Copies globals to property suite */

void GlobalsToProperties(GPtr globals)
{
	short 	loop = 0;
	PIType	key = NULLID;
	Handle	complex = NULL;
	
	for (loop = kProp1; loop <= kPropEnd; loop++)
		{
			key = GetPropKey(loop);
			switch (loop)
			{
				case iPropTitle:
					break; // nuthin
				case iPropSerialString:
					break; // nuthin
				case iPropRulerUnits:
					break; // nuthin
				case iPropBigNudgeH:
					PutSimple(key, PIDoubleToFixed(gNudgeH, kHard16));
					break;
				case iPropBigNudgeV:
					PutSimple(key, PIDoubleToFixed(gNudgeV, kHard16));
					break;
				case iPropRulerOriginH:
					PutSimple(key, PIDoubleToFixed(gOriginH, kSoft16));
					break;
				case iPropRulerOriginV:
					PutSimple(key, PIDoubleToFixed(gOriginV, kSoft16));
					break;
				case iPropGridMajor:
					PutSimple(key, PIDoubleToFixed(gGridMajor, kSoft16));
					break;
				case iPropGridMinor:
					PutSimple(key, gGridMinor);
					break;
				case iPropCopyright:
					PutSimple(key, gCopyright);
					break;
				case iPropURL:
					complex = PIString2Handle(gURL);
					if (complex != NULL)
					{ // got a handle.
						PutComplex(key, 0, complex);
						PIDisposeHandle(complex);
						complex = NULL;
					}
					break;
			}
		}
}
/*****************************************************************************/
/* Copies property suite information to globals */

void PropertiesToGlobals(GPtr globals)
{
	short 	loop = 0;
	long	simple = 0;
	double	simpleDouble = 0;
	Handle	complex = NULL;
	OSType	key = NULLID;
	
	for (loop = kProp1; loop <= kPropEnd; loop++)
		{
			key = GetPropKey(loop);	
			switch (loop)
			{
				case iPropTitle:
					break; // nuthin
				case iPropSerialString:
					break; // nuthin
				case iPropRulerUnits:
					break; // nuthin
				case iPropBigNudgeH:
					GetSimple(key,&simple);
					simpleDouble = PIFixedToDouble(simple, kHard16);
					gNudgeH = simpleDouble;
					break;
				case iPropBigNudgeV:
					GetSimple(key,&simple);
					simpleDouble = PIFixedToDouble(simple, kHard16);
					gNudgeV = simpleDouble;
					break;
				case iPropRulerOriginH:
					GetSimple(key,&simple);
					simpleDouble = PIFixedToDouble(simple, kSoft16);
					gOriginH = simpleDouble;
					break;
				case iPropRulerOriginV:
					GetSimple(key,&simple);
					simpleDouble = PIFixedToDouble(simple, kSoft16);
					gOriginV = simpleDouble;
					break;
				case iPropGridMajor:
					GetSimple(key,&simple);
					simpleDouble = PIFixedToDouble(simple, kSoft16);
					gGridMajor = simpleDouble;
					break;
				case iPropGridMinor:
					GetSimple(key,&simple);
					gGridMinor = simple;
					break;
				case iPropCopyright:
					GetSimple(key,&simple);
					gCopyright = simple;
					break;
				case iPropURL:
					GetComplex(key, 0, &complex);
					PIHandle2String(complex, gURL);
					if (complex != NULL) PIDisposeHandle(complex);
					complex = NULL;
					break;
			}
		}
}

/*****************************************************************************/
