/*
	File: PropetizerUIWin.c

	Copyright 1996, Adobe Systems Incorporated.
	All rights reserved.

	C source file for Windows specific code for Propetizer example.
*/

#include "Propetizer.h"

/****************************************************************************/

void	ShowOutputBuffer(GPtr globals, HWND hPrev);
short Validate (HWND hDlg, short *item, double *dx, Boolean useAlert);
void UpdateDialogInfo (GPtr globals, HWND dp, Boolean UseProps);
BOOL WINAPI FilterProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam);
void DoThis (GPtr globals);

/*****************************************************************************/

/* Calculates the rectangle we will want to use for the proxy for an image. */

void GetProxyItemRect (HWND	hDlg, Rect *proxyItemRect)
	{

	RECT	wRect;
	
	GetWindowRect(GetDlgItem(hDlg, proxyItem), &wRect);	
	ScreenToClient (hDlg, (LPPOINT)&wRect);
	ScreenToClient (hDlg, (LPPOINT)&(wRect.right));

	proxyItemRect->left = (short)wRect.left;
	proxyItemRect->top  = (short)wRect.top;
	proxyItemRect->right = (short)wRect.right;
	proxyItemRect->bottom = (short)wRect.bottom;
	
	}

/*****************************************************************************/

void DoAbout (GPtr globals)
{
	ShowAbout((AboutRecordPtr)gStuff, hDllInstance, AboutID);
}

/*****************************************************************************/

void ShowOutputBuffer(GPtr globals, HWND hDlg)
{
	PSPixelMap pixels;
	PSPixelMask mask;
	PAINTSTRUCT  ps;
	Rect  itemBounds;
	RECT  wRect;
	POINT	mapOrigin; 
	HDC		hDC;

	GetWindowRect(GetDlgItem(hDlg, proxyItem), &wRect);
	mapOrigin.x = 0;
	mapOrigin.y = 0;
	ClientToScreen(hDlg, &mapOrigin);
	
	itemBounds.left = (short)(((wRect.right + wRect.left - gStuff->outRect.right + gStuff->outRect.left) / 2) - mapOrigin.x);
	itemBounds.top = (short)(((wRect.bottom + wRect.top - gStuff->outRect.bottom + gStuff->outRect.top) / 2) - mapOrigin.y);
	itemBounds.right = (short)(itemBounds.left + (gStuff->outRect.right - gStuff->outRect.left));
	itemBounds.bottom = (short)(itemBounds.top + (gStuff->outRect.bottom - gStuff->outRect.top));
	
	hDC = BeginPaint(hDlg, &ps);	

	wRect.left = itemBounds.left;
	wRect.top = itemBounds.top;
	wRect.right = itemBounds.right;
	wRect.bottom = itemBounds.bottom;

	InflateRect(&wRect, kGutter, kGutter);
	//FillRect(hDC, &wRect, (HBRUSH) (COLOR_WINDOW));
	FrameRect(hDC, &wRect, GetStockObject(BLACK_BRUSH));	
	InflateRect(&wRect, -kGutter, -kGutter);
	
	pixels.version = 1;
	pixels.bounds.top = gStuff->outRect.top;
	pixels.bounds.left = gStuff->outRect.left;
	pixels.bounds.bottom = gStuff->outRect.bottom;
	pixels.bounds.right = gStuff->outRect.right;
	pixels.imageMode = gStuff->imageMode;
	pixels.rowBytes = gStuff->outRowBytes;
	pixels.colBytes = gStuff->outHiPlane - gStuff->outLoPlane + 1;
	pixels.planeBytes = 1;
	pixels.baseAddr = gStuff->outData;

	pixels.mat = NULL;
	pixels.masks = NULL;
	pixels.maskPhaseRow = 0;
	pixels.maskPhaseCol = 0;

	if (gStuff->isFloating) {
		mask.next = NULL;
		mask.maskData = gStuff->maskData;
		mask.rowBytes = gStuff->maskRowBytes;
		mask.colBytes = 1;
		mask.maskDescription = kSimplePSMask;
	
		pixels.masks = &mask;
	} else if ((gStuff->inLayerPlanes != 0) && (gStuff->inTransparencyMask != 0)) {
		mask.next = NULL;
		mask.maskData = ((char *) gStuff->outData) + (gStuff->outHiPlane - gStuff->outLoPlane);
		mask.rowBytes = gStuff->outRowBytes;
		mask.colBytes = gStuff->outHiPlane - gStuff->outLoPlane + 1;
		mask.maskDescription = kSimplePSMask;
	
		pixels.masks = &mask;
	}

	(gStuff->displayPixels)(&pixels, &pixels.bounds, itemBounds.top, itemBounds.left, (void *)hDC);

	EndPaint(hDlg, (LPPAINTSTRUCT) &ps);
}

/*****************************************************************************/
// Loops through values and automatically grabs and validates item.  Returns
// noErr if okay, otherwise focus has been forced back to invalid item.

short Validate(HWND dp, short *item, double *dx, Boolean useAlert)
{
	short 	numberErr = noErr;
	long 	x = 0; // returned long
	short 	loop = 1;
	
	const double propDoubles[] = {
		iPropBigNudgeH, kNudgeMin, kNudgeMax,
		iPropBigNudgeV, kNudgeMin, kNudgeMax,
		iPropRulerOriginH, kOriginMin, kOriginMax,
		iPropRulerOriginV, kOriginMin, kOriginMax,
		iPropGridMajor, kGridMin, kGridMax
		};

	const short	numPropDoubles = 5;

	const long	propNumbers[] = {
		iPropGridMinor, kGridMin, kGridMax,
		iPropCopyright, kCopyMin, kCopyMax
		};

	const short	numPropNumbers = 2;
	
	const short foundIt = -1; // flag that we found it

	
	// first loop through doubles, looking for match
	while (loop <= numPropDoubles && loop != foundIt)
	{
		if (propDoubles[loop*3-3] == *item)
		{
			numberErr = FetchDouble (dp,
									 *item,
									 propDoubles[loop*3-2], // max
									 propDoubles[loop*3-1], // min
									 dx); // returned double
			if (numberErr != noErr && useAlert)
			{ // out of bounds problem.  Report and change focus back.
				AlertDouble (dp,
							 *item,
							 propDoubles[loop*3-2], // max
							 propDoubles[loop*3-1], // min
							 dx, // value
							 hDllInstance, // global ref for win
							 AlertID, // global alert dialog
							 numberErr); // error we found
				*item = 0;
			}
			loop = foundIt; // found it!
		} else loop++; // next
	}

	if (loop != foundIt) loop = 1; // reset
	// now loop through longs, looking for match
	while (loop <= numPropNumbers && loop != foundIt)
	{
		if (propNumbers[loop*3-3] == *item)
		{
			numberErr = FetchNumber (dp,
									 *item,
									 propNumbers[loop*3-2], // max
									 propNumbers[loop*3-1], // min
									 &x); // returned long
			if (numberErr != noErr && useAlert)
			{ // out of bounds problem.  Report and change focus back.
				AlertNumber (dp,
							 *item,
							 propNumbers[loop*3-2], // max
							 propNumbers[loop*3-1], // min
							 &x, // value
							 hDllInstance, // global ref for win
							 AlertID, // global alert dialog
							 numberErr); // error we found
				*item = 0;
			}
			*dx = (double)x; // coerce long as returned double
			loop = foundIt; // found it!
		} else loop++; // next
	}
	return numberErr;
}

/*****************************************************************************/

BOOL WINAPI FilterProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)       // Win32 Change
{
	long				x = 1;
	double				dx = 0, x2 = 0;
	short				numberErr = noErr;
	int32				simple;
	Handle				complex;
	static double		cPropBigNudgeH, cPropBigNudgeV,
						cPropRulerOriginH, cPropRulerOriginV,
						cPropGridMajor;
	static long			cPropGridMinor, cCopyright;
	static Str255		cURL = "";
	short				item, loop;
	static short		lastItem = 0, lastTitem = 0;
	Str255				ourText;
	short				gotErr = 0;
	PIType				key = NULLID;
	RECT				imageRect;
	int					cmd;
	static GPtr			globals = NULL;		  /* need to be static */

	switch  (wMsg)
	{

		case  WM_INITDIALOG:

			/* set up globals	*/

			globals = (GPtr) lParam;

			CenterDialog(hDlg);
			
			GetProxyItemRect (hDlg, &gProxyRect); 
			
			gOrigProxyRect = gProxyRect;

			CalcGlobalProxies (globals);
			SetupFilterRecordForProxy (globals, gProxyScaleRect, 1);
			
			UpdateDialogInfo(globals, hDlg, false);
			cPropBigNudgeH = gNudgeH;
			cPropBigNudgeV = gNudgeV;
			cPropRulerOriginH = gOriginH;
			cPropRulerOriginV = gOriginV;
			cPropGridMajor = gGridMajor;
			cPropGridMinor = gGridMinor;
			cCopyright = gCopyright;
			AppendString(cURL, gURL, 1, gURL[0]);

			NumToString((long)(gRepFactor*100), ourText);
			ourText[++ourText[0]] = '%';
			StuffText(hDlg, iRepFactor, ourText);

			CalcProxyScaleFactor(gProxyScaleRect, &gStuff->outRect, &loop);

			DoZoomRect (globals);

			SelectTextItem(hDlg, kPropStart);

			/* drop into PAINT */
		case WM_PAINT:
			ShowOutputBuffer(globals, hDlg);	 // Draw Proxy Item
			return FALSE;
			break;

		case  WM_COMMAND:
	  		item = COMMANDID (wParam);              // WIN32 Change
			cmd = HIWORD (wParam);

			switch  (item)
			{
				case iPropURL:
					if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
					{ // Didn't need to change focus back.  Process.
						if (cmd == EN_KILLFOCUS)
							lastItem = item; // validate will handle
						else if (cmd == EN_CHANGE)
							FetchText(hDlg, item, cURL);
					}
					break;
				case iPropBigNudgeH:
				case iPropBigNudgeV:
				case iPropRulerOriginH:
				case iPropRulerOriginV:
				case iPropGridMajor:
				case iPropGridMinor:
				case iPropCopyright:
					if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
					{ // Didn't need to reset focus.  Process.
						if (cmd == EN_KILLFOCUS)
							lastItem = item; // validate will get this
						else if (cmd == EN_SETFOCUS)
							lastTitem = item; // last text item hilited
						else if (cmd == EN_CHANGE)
						{ // give me the number, only, not the alert
							numberErr = Validate(hDlg, &item, &dx, false);
							switch (item)
							{
								case iPropBigNudgeH:
									cPropBigNudgeH = dx;
									break;
								case iPropBigNudgeV:
									cPropBigNudgeV = dx;
									break;
								case iPropRulerOriginH:
									cPropRulerOriginH = dx;
									break;
								case iPropRulerOriginV:
									cPropRulerOriginV = dx;
									break;
								case iPropGridMajor:
									cPropGridMajor = dx;
									break;
								case iPropGridMinor:
									cPropGridMinor = (long)dx;
									break;
								case iPropCopyright:
									cCopyright = (long)dx;
									break;
							} // end switch
						} // end en_change
					} // numberErr == err
					return FALSE; // handled it
					break;
				case plusButton:
				case minusButton:
					if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
					{ // didn't have to change focus.  Continue.
						if (cmd == BN_CLICKED)
						{
							if (item == plusButton)
								ZoomIn (&gZoomFactor);
							else ZoomOut (&gZoomFactor);
							gotErr = GetZoomFactor(globals);

							EnableControl(hDlg, minusButton);
							EnableControl(hDlg, plusButton);

							if (gotErr == hitLowBounds)
								DisableControl(hDlg, minusButton);
							else if (gotErr == hitHighBounds)
								DisableControl(hDlg, plusButton);

							NumToString((long)(gRepFactor*100), ourText);
							ourText[++ourText[0]] = '%';
							StuffText(hDlg, iRepFactor, ourText);

							(void) DoZoomRect (globals);
			
							// Invalidate Proxy Item
							GetWindowRect(GetDlgItem(hDlg, proxyItem), &imageRect);
							
							// change the rect here, if resized
							
							ScreenToClient (hDlg, (LPPOINT)&imageRect);
							ScreenToClient (hDlg, (LPPOINT)&(imageRect.right));
							InvalidateRect (hDlg, &imageRect, FALSE);
							return FALSE;
						} // btn wasn't clicked
					} // had to change focus back.
					break;
				case updateButton:
					if (cmd == BN_CLICKED)
					{ // clicked me.  Get the properties and populate.
						UpdateDialogInfo(globals, hDlg, true);
						DoThis (globals); // ^^^^^^^
						for (loop = kProp1; loop <= kPropEnd; loop++)
						{
							key = GetPropKey (loop);
							switch (loop)
							{
								case iPropBigNudgeH:
									GetSimple(key, &simple);
									cPropBigNudgeH = PIFixedToDouble(simple, kHard16);
									break;
								case iPropBigNudgeV:
									GetSimple(key, &simple);
									cPropBigNudgeV = PIFixedToDouble(simple, kHard16);
									break;
								case iPropRulerOriginH:
									GetSimple(key, &simple);
									cPropRulerOriginH = PIFixedToDouble(simple, kSoft16);
									break;
								case iPropRulerOriginV:
									GetSimple(key, &simple);
									cPropRulerOriginV = PIFixedToDouble(simple, kSoft16);
									break;
								case iPropGridMajor:
									GetSimple(key, &simple);
									cPropGridMajor = PIFixedToDouble(simple, kSoft16);
									break;
								case iPropGridMinor:
									GetSimple(key, &simple);
									cPropGridMinor = simple;
									break;
								case iPropCopyright:
									GetSimple(key, &simple);
									cCopyright = simple;
									break;
								case iPropURL:
									GetComplex(key, 0, &complex);
									PIHandle2String(complex, cURL);
									if (complex != NULL) PIDisposeHandle(complex);
									complex = NULL;
									break;
							} // end switch
						} // end for
						SendDlgItemMessage(hDlg, updateButton, BM_SETSTATE,
									   0, 0);
						SelectTextItem(hDlg, lastTitem);
						lastItem = 0;
						return FALSE; // handled
					} // don't have to validate cuz we're overriding all
					break;
				case revertButton:
					if (cmd == BN_CLICKED)
					{ // clicked me!  Revert.
						UpdateDialogInfo(globals, hDlg, false);
						cPropBigNudgeH = gNudgeH;
						cPropBigNudgeV = gNudgeV;
						cPropRulerOriginH = gOriginH;
						cPropRulerOriginV = gOriginV;
						cPropGridMajor = gGridMajor;
						cPropGridMinor = gGridMinor;
						cCopyright = gCopyright;
						cURL[ (cURL[0] = 0)+1 ] = 0;
						AppendString(cURL, gURL, 1, gURL[0]);
						SendDlgItemMessage(hDlg, updateButton, BM_SETSTATE,
									   0, 0);
						SelectTextItem(hDlg, lastTitem);
						lastItem = 0;
						return FALSE; // handled
						break;
					} // don't have to validate cuz we're overriding all
					break;
				case proxyItem:
					if (cmd == BN_CLICKED)
					{ // went to move proxy item.  Check stuff.
						if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
						{ // didn't have to change focus.  Continue.
							// move proxy here
							return FALSE; // handled
						}
					} // didn't click on proxy
					break;
				case ok:
					if ((numberErr = Validate(hDlg, &lastItem, &dx, true)) == noErr)
					{ // validated fine
						if (cmd == BN_CLICKED)
						{ // wants to dismiss.  Transfer all locals to globals.
							gNudgeH = cPropBigNudgeH;
							gNudgeV = cPropBigNudgeV;
							gOriginH = cPropRulerOriginH;
							gOriginV = cPropRulerOriginV;
							gGridMajor = cPropGridMajor;
							gGridMinor = cPropGridMinor;
							gCopyright = cCopyright;
							gURL[ (gURL[0] = 0)+1 ] = 0;
							AppendString(gURL, cURL, 1, cURL[0]);
							EndDialog(hDlg, item); 
							return TRUE;
						}
					}
					break;
				case cancel:
					if (cmd == BN_CLICKED)
					{
						gResult = userCanceledErr;
						EndDialog(hDlg, item);          // WIN32 change
						return TRUE;
					}
					// don't ever validate during cancel
					break;
				default:
					break;
			} // switch (item)
			break; // case WM_COMMAND
		default:
			return FALSE;
			break;
		} // switch

	return  TRUE;
}

/*****************************************************************************/
short PromptUserForInput (GPtr globals)
{

	int    nResult;
	PlatformData *platform;

	platform = ((FilterRecordPtr) gStuff)->platformData;

	/* Query the user for percent dissolve. */

	nResult = DialogBoxParam(hDllInstance,
							 (LPSTR)"PROPETIZERPARAM",
							 (HWND)platform->hwnd,
							 (FARPROC)FilterProc,
							 (LPARAM)globals);

	return (nResult == ok);

}	

/****************************************************************************/
/* Example for ShowAlert() function which takes a string ID as parameter    */
/* and displays a message box                                               */
/****************************************************************************/

short ShowAlert (short stringID)
{
	char szMessage[256];
	char szTitle[128];

	LoadString(hDllInstance, stringID, szMessage, sizeof szMessage);
	LoadString(hDllInstance, 2, szTitle, sizeof szTitle);
	return  ( MessageBox((HWND)NULL, (LPSTR)szMessage, (LPSTR)szTitle, MB_OK | MB_ICONHAND) );

}

/*****************************************************************************/
// This grabs properties from the suite or from the globals and populates the
// dialog with them.

void UpdateDialogInfo (GPtr globals, HWND dp, Boolean UseProps)
{
	short		loop = 0;
	long		simple = 0;
	double		simpleDouble = 0;
	Handle		complex = NULL;
	Str255		ourText = "";
	OSType		key = NULLID;
	
	for (loop = kProp1; loop <= kPropEnd; loop++)
	{
		ourText[ (ourText[0] = 0)+1 ] = 0;
		key = GetPropKey(loop);
		switch (loop)
		{

			case iPropTitle:
				GetComplex(propTitle, 0, &complex);
				PIHandle2String(complex, ourText);
				StuffText(dp, loop, ourText);
				if (complex != NULL) PIDisposeHandle(complex);
				complex = NULL;
				break;
			case iPropSerialString:
				GetComplex(propSerialString, 0, &complex);
				PIHandle2String(complex, ourText);
				StuffText(dp, loop, ourText);
				if (complex != NULL) PIDisposeHandle(complex);
				complex = NULL;
				break;
			case iPropRulerUnits:
				GetSimple(key, &simple );
				PIGetString (ourText, kUnits+simple);
				StuffText(dp, loop, ourText);
				break;
			case iPropBigNudgeH:
				if (UseProps)
				{
					GetSimple(key, &simple);
					simpleDouble = PIFixedToDouble(simple, kHard16);
				}
				else
					simpleDouble = gNudgeH;
				StuffDouble(dp, loop, simpleDouble, kPrecision);
				break;
			case iPropBigNudgeV:
				if (UseProps)
				{
					GetSimple(key, &simple);
					simpleDouble = PIFixedToDouble(simple, kHard16);
				}
				else
					simpleDouble = gNudgeV;
				StuffDouble(dp, loop, simpleDouble, kPrecision);
				break;
			case iPropRulerOriginH:
				if (UseProps)
				{
					GetSimple(key, &simple);
					simpleDouble = PIFixedToDouble(simple, kSoft16);
				}
				else
					simpleDouble = gOriginH;
				StuffDouble(dp, loop, simpleDouble, kPrecision);
				break;
			case iPropRulerOriginV:
				if (UseProps)
				{
					GetSimple(key, &simple);
					simpleDouble = PIFixedToDouble(simple, kSoft16);
				}
				else
					simpleDouble = gOriginV;
				StuffDouble(dp, loop, simpleDouble, kPrecision);
				break;
			case iPropGridMajor:
				if (UseProps)
				{
					GetSimple(key, &simple);
					simpleDouble = PIFixedToDouble(simple, kSoft16);
				}
				else
					simpleDouble = gGridMajor;
				StuffDouble(dp, loop, simpleDouble, kPrecision);
				break;
			case iPropGridMinor:
				if (UseProps)
					GetSimple(key, &simple);
				else
					simple = gGridMinor;
				StuffNumber(dp, loop, simple);
				break;
			case iPropCopyright:
				if (UseProps)
					GetSimple(key, &simple);
				else
					simple = gCopyright;
				StuffNumber(dp, loop, simple);
				break;
			case iPropURL:
				if (UseProps)
				{
					GetComplex(key, 0, &complex);
					PIHandle2String(complex, ourText);
					StuffText(dp, loop, ourText);
					if (complex != NULL) PIDisposeHandle(complex);
					complex = NULL;
				}
				else
					StuffText(dp, loop, gURL);
				break;
		}
	}
}

/*****************************************************************************/

/* Initialization and termination code for window's dlls. */

/* ------------------------------------------------
 *   Code from Borland's window's dll example code.
 * ------------------------------------------------
 */
#if defined(__BORLANDC__)
// Turn off warning: Parameter '' is never used; effects next function only
#pragma argsused
#endif

#ifdef WIN32

// Every 32-Bit DLL has an entry point DLLInit

BOOL APIENTRY DLLInit(HANDLE hInstance, DWORD fdwReason, LPVOID lpReserved)
{

	if (fdwReason == DLL_PROCESS_ATTACH)
		hDllInstance = hInstance;

	return TRUE;   // Indicate that the DLL was initialized successfully.
}

#else
// Every 16-Bit DLL has an entry point LibMain and an exit point WEP.
int FAR PASCAL LibMain( HANDLE hInstance, WORD wDataSegment,
								   WORD wHeapSize, LPSTR lpszCmdLine )
{
	// Required when using Zortech; causes blink to include startup code
	extern __acrtused_dll;

	// The startup code for the DLL initializes the local heap (if there is one)
	// with a call to LocalInit which locks the data segment.
	if ( wHeapSize != 0 )
		UnlockData( 0 );

	hDllInstance = hInstance;
	return 1;   // Indicate that the DLL was initialized successfully.
}

int FAR PASCAL WEP(int nParam)
{
	switch  (nParam) {
	  case  WEP_SYSTEM_EXIT: // System shutdown in progress
	  case  WEP_FREE_DLL   : // DLL use count is 0
	  default :              // Undefined;  ignore
			return  1;
	}
}
#endif
/*****************************************************************************/

void DoThis (GPtr globals)
{
	// ResType	signature;
    DWORD		lpdwHandle; 	// pointer to variable to receive zero
    char		*lpBuffer[80];	// address of buffer for version pointer
    char		lpFilename[256];
	int32		len = 0;
	char		lpData[1024];
	VS_FIXEDFILEINFO	*ffi;
	DWORD		x = 0;
	Ptr			p = 0;

	len = GetModuleFileName(NULL /* hDllInstance */, lpFilename, 255);
	len = GetFileVersionInfoSize(lpFilename, &lpdwHandle);

	GetFileVersionInfo(lpFilename, lpdwHandle, len, lpData);

	x = VerQueryValue(lpData, TEXT("\\"), lpBuffer, &len);
	
	p = *(lpBuffer);
	ffi = (VS_FIXEDFILEINFO *)p;

	x = ffi->dwSignature; // 0xFEEFO4BD
	x++;
	x = ffi->dwFileVersionMS;
	x++;
	x = ffi->dwFileVersionLS;
	x++;
	x = ffi->dwFileFlags; // release, prerelease, etc.
	x++;

/*
	VS_VERSION_INFO { 
    WORD  wLength; 
    WORD  wValueLength; 
    WORD  bText; 
    WCHAR szKey[]; 
    BYTE  Padding1[]; 
    VS_FIXEDFILEINFO Value; 
    BYTE  Padding2[]; 
    BYTE  Children[]; }; 
	typedef struct _VS_FIXEDFILEINFO {  // vsffi 
    DWORD dwSignature; 
    DWORD dwStrucVersion; 
    DWORD dwFileVersionMS; 
    DWORD dwFileVersionLS; 
    DWORD dwProductVersionMS; 
    DWORD dwProductVersionLS; 
    DWORD dwFileFlagsMask; 
    DWORD dwFileFlags; 
    DWORD dwFileOS; 
    DWORD dwFileType; 
    DWORD dwFileSubtype; 
    DWORD dwFileDateMS; 
    DWORD dwFileDateLS; 
} VS_FIXEDFILEINFO; */

  
	x = VerQueryValue(lpData, 
				  TEXT("\\StringFileInfo\\040904E4\\OriginalFilename"),
				  lpBuffer,
				  &len);

	x = VerQueryValue(lpData,
				  TEXT("\\StringFileInfo\\040904E4\\FileVersion"),
				  lpBuffer,
				  &len);
	
}